// ********************************************************************************************************************
//  Header file for generic Clough functions
// ********************************************************************************************************************

#ifndef __CLOUGH_MCU_H__
#define __CLOUGH_MCU_H__

// ********************************************************************************************************************
// Number of IRQ priority level bits - must be defined before loading core_cm0.h
// ********************************************************************************************************************

#define __NVIC_PRIO_BITS 2

// ********************************************************************************************************************
// Includes
// ********************************************************************************************************************
#include "core_cm0.h"
#include <stdint.h>

// ********************************************************************************************************************
// Base addresses
// ********************************************************************************************************************

#define FLASH_BASE       ((uint32_t) 0x00000000)
#define SRAM_BASE        ((uint32_t) 0x20000000)
#define SRAM_TOP         ((uint32_t) 0x20001FFF)
#define PERIPH_BASE      ((uint32_t) 0x40000000)
#define SDIO_BASE        ((uint32_t) 0x50000000)
#define ASIC_7B_BASE     ((uint32_t) 0x50000000)
#define BTE_BASE         ((uint32_t) 0x50000080)
#define ASIC_16B_BASE    ((uint32_t) 0x50010000)
#define TIMER_BASE       ((uint32_t) 0x50020000)
#define FLASH_CTRL_BASE  ((uint32_t) 0x50020020)

#define FLASH_1ST_SECTOR (FLASH_BASE + 0x40000)
#define FLASH_2ND_SECTOR (FLASH_BASE + 0x40100)

// ********************************************************************************************************************
// Peripheral addresses
// ********************************************************************************************************************

// Flash programming addresses
#define FLASH_ACCESS_ADDR         ((volatile uint32_t *) (FLASH_CTRL_BASE  + 0x00))
#define FLASH_PROGRESS_FLAGS      ((volatile uint8_t  *) (FLASH_CTRL_BASE  + 0x01))
#define FLASH_WRITE_DATA          ((volatile uint8_t  *) (FLASH_CTRL_BASE  + 0x04))
#define FLASH_UNLOCK_BYTE_WRITE   ((volatile uint32_t *) (FLASH_CTRL_BASE  + 0x08))
#define FLASH_BYTE_WRITE          ((volatile uint32_t *) (FLASH_CTRL_BASE  + 0x0C))
#define FLASH_UNLOCK_SECTOR_ERASE ((volatile uint32_t *) (FLASH_CTRL_BASE  + 0x10))
#define FLASH_SECTOR_ERASE        ((volatile uint32_t *) (FLASH_CTRL_BASE  + 0x14))
#define FLASH_UNLOCK_BLOCK_ERASE  ((volatile uint32_t *) (FLASH_CTRL_BASE  + 0x18))
#define FLASH_BLOCK_ERASE         ((volatile uint32_t *) (FLASH_CTRL_BASE  + 0x1C))
#define FLASH_ACCESS_CFG          ((volatile uint32_t *) (FLASH_CTRL_BASE  + 0x20))
#define CODE_PROT_REG             ((volatile uint32_t *) (FLASH_CTRL_BASE  + 0x24))
#define CODE_PROT_PAT             ((volatile uint32_t *) (FLASH_CTRL_BASE  + 0x24))

// Flash config words
#define FLASH_CONFIG_W0           ((volatile uint32_t *) (FLASH_2ND_SECTOR + 0x00))
#define FLASH_CONFIG_W1           ((volatile uint32_t *) (FLASH_2ND_SECTOR + 0x04))

// Special AyDeeKay flash sector
#define FLASH_ADK_SECTOR          ((uint32_t) (FLASH_BASE + 0x27F00 ))

// Block transfer engine
#define BTE_CFG                ((volatile uint32_t *) (BTE_BASE + 0x00))
#define BTE_SFR_ADDR           ((volatile uint16_t *) (BTE_BASE + 0x00))
#define BTE_NUM_XFER           ((volatile uint8_t  *) (BTE_BASE + 0x02))
#define BTE_CMD                ((volatile uint8_t  *) (BTE_BASE + 0x03))
#define BTE_SRAM_ADDR          ((volatile uint32_t *) (BTE_BASE + 0x04))

// General purpose timers

typedef struct {
  __IO int32_t  COUNT;
  __IO uint8_t  CFG;
} Timer_Type;

typedef struct {
  __IO uint8_t  CFG;
  __IO uint8_t  empty1;
  __IO uint8_t  empty2;
  __IO uint8_t  empty8;
  __O  uint32_t KEY;
} Watchdog_Type;

#define TIMER0                 ((Timer_Type *) (TIMER_BASE + 0x00))
#define TIMER1                 ((Timer_Type *) (TIMER_BASE + 0x08))
#define TIMER2                 ((Timer_Type *) (TIMER_BASE + 0x0C))

//#define TIMER0_VAL            ((volatile int32_t  *) (TIMER_BASE + 0x00))
//#define TIMER0_CFG            ((volatile uint8_t  *) (TIMER_BASE + 0x04))
//#define TIMER1_VAL            ((volatile int32_t  *) (TIMER_BASE + 0x08))
//#define TIMER1_CFG            ((volatile uint8_t  *) (TIMER_BASE + 0x0C))
//#define TIMER2_VAL            ((volatile int32_t  *) (TIMER_BASE + 0x10))
//#define TIMER2_CFG            ((volatile uint8_t  *) (TIMER_BASE + 0x14))

// Watchdog timer
#define WATCHDOG              ((__IO Watchdog_Type *) (TIMER_BASE + 0x18))
//#define WATCHDOG_CFG_REG      ((volatile uint8_t  *) (TIMER_BASE + 0x18))
//#define WATCHDOG_CLR_REG      ((volatile uint32_t *) (TIMER_BASE + 0x1C))


// ********************************************************************************************************************
// Key Words
// ********************************************************************************************************************

// Flash unlock values
#define FLASH_UNLOCK_BYTE_WRITE_KEY      ((uint32_t) 0x55555555)
#define FLASH_BYTE_WRITE_KEY             ((uint32_t) 0xAAAAAAAA)
#define FLASH_UNLOCK_BLOCK_ERASE_KEY     ((uint32_t) 0x77777777)
#define FLASH_BLOCK_ERASE_KEY            ((uint32_t) 0x88888888)
#define FLASH_UNLOCK_SECTOR_ERASE_KEY    ((uint32_t) 0x66666666)
#define FLASH_SECTOR_ERASE_KEY           ((uint32_t) 0x99999999)
#define FLASH_ADKBYTE_WRITE_KEY          ((uint32_t) 0x502901FF)
#define FLASH_ADKSECTOR_ERASE_KEY        ((uint32_t) 0x502901FF)

// Code protection lock value
#define CODE_PROTECT_KEY                 ((uint32_t) 0xF2E11047)

// Watchdog
#define WATCHDOG_KEY0         ((uint32_t) 0x3C570001)
#define WATCHDOG_KEY1         ((uint32_t) 0x007F4AD6)
#define WATCHDOG_KEY_CLEAR    ((uint32_t) 0x00000000)


// ********************************************************************************************************************
// Bit indexing helpers
// ********************************************************************************************************************

// Flash progress flags
#define FLASH_WRITE_IN_PROGRESS          ((uint8_t) 0x01)
#define FLASH_SECTOR_ERASE_IN_PROGRESS   ((uint8_t) 0x02)
#define FLASH_BLOCK_ERASE_IN_PROGRESS    ((uint8_t) 0x04)

// Block transfer engine (bit masks for BTE_CMD)
#define BTE_START_CMD          (((uint8_t) 1) << 3)
#define BTE_BLOCKING_XFER_CMD  (((uint8_t) 1) << 2)
#define BTE_WRITE_CMD          (((uint8_t) 1) << 1)
#define BTE_ADDR_INCR_CMD      (((uint8_t) 1) << 0)

// ********************************************************************************************************************
// Include CortexM0 core functions.  Must be done after interrupt enums.
// ********************************************************************************************************************

// #include "core_cm0.h"

#endif
