/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file adc_device.h
 */

#ifndef __ADC_DEVICE_H__
#define __ADC_DEVICE_H__

#include <stdint.h>
#include "hdf.h"

/**
 * @brief A structure to represent ADC device
 */
typedef const struct ADC_Device_t {
    void (*SelectChannel)(uint8_t channel);
    void (*ClkDiv)(uint8_t div);
    void (*SampleCycle)(uint8_t val);
    uint8_t (*Read)(void);
    void (*Start)(void);
    void (*SetReferenceHigh)(uint8_t val);
    void (*SetReferenceLow)(uint8_t val);
    void (*SetReferenceGain)(uint8_t val);
    uint8_t (*ConversionInProgress)(void);
    void (*SetRefSourceVDD)(void);
    void (*SetRefSourceBandGap)(void);
    void (*SetStreamingMode)(void);
    void (*SetSingleMode)(void);
    void (*SetCalibrationMode)(void);
    void (*SetNormalMode)(void);
    const char *Name;
} ADC_Device_t;

/**
 * @brief The enumeration of ADC mode
 */
#define ADC_MODE_STREAM                 ((uint32_t)(1<<0))
#define ADC_MODE_SINGLE                 ((uint32_t)(1<<1))
#define ADC_MODE_NORMAL                 ((uint32_t)(1<<2))
#define ADC_MODE_CALIBRATION            ((uint32_t)(1<<3))
#define ADC_MODE_REF_VDD                ((uint32_t)(1<<4))
#define ADC_MODE_REF_BANDGAP            ((uint32_t)(1<<5))

/**
 * @brief Select ADC channel for a ADC device
 *
 * @param dev The pointer to ADC device
 * @param channel The enumeration of ADC channel.
 * return 0 for success or error code upon a failure.
 */
int32_t ADC_devSelectChannel(ADC_Device_t *dev, uint8_t channel);

/**
 * @brief Read ADC result from a ADC device
 *
 * @param dev The pointer to ADC device
 * @param val The pointer to the result.
 * @return 0 for success or error code upon a failure.
 */
int32_t ADC_devRead(ADC_Device_t *dev, uint8_t *val);

/**
 * @brief Start a ADC conversion, wait for its completion and read result back.
 *
 * @param dev The pointer to ADC device
 * @param channel The enumeration of ADC channel.
 * @param val The pointer to the result.
 * @return 0 for success or error code upon a failure.
 */
int32_t ADC_devTake(ADC_Device_t *dev, uint8_t channel, uint8_t *val);

/**
 * @brief Start a ADC conversion.
 *
 * @param dev The pointer to ADC device
 * @return 0 for success or error code upon a failure.
 */
int32_t ADC_devStart(ADC_Device_t *dev);

/**
 * @brief Check status of a ADC conversion.
 *
 * @param dev The pointer to ADC device
 * @return 1 for busy, 0 for idle or error code upon a failure.
 */
int32_t ADC_devConversionInProgress(ADC_Device_t *dev);

/**
 * @brief Initialize ADC reference settings. It sets the dynamic range and the gain.
 *
 * @param dev The pointer to ADC device
 * @param refh The high watermark to the reference voltage.
 * @param refl The low watermark to the reference voltage.
 * @param refg The gain of the reference voltage.
 * @return 0 for success or error code upon a failure.
 */
int32_t ADC_devConfigReference(ADC_Device_t *dev, uint8_t refh, uint8_t refl, uint8_t refg);

/**
 * @brief Initialize ADC device.
 *
 * @param dev The pointer to ADC device
 * @param div The clock divider.
 * @param cycle The ADC cycle.
 * @param mode The ADC mode.
 * @return 0 for success or error code upon a failure.
 */
int32_t ADC_devInit(ADC_Device_t *dev, uint8_t div, uint8_t cycle, uint32_t mode);

/**
 * @brief ADC Request Device
 *
 * @param name The name string of ADC device.
 * @return The pointer to the ADC device instance.
 */
ADC_Device_t *ADC_devRequest(const char *name);

#endif /* __ADC_DEVICE_H__ */