/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file pmu_device.h
 */

#ifndef __PMU_DEVICE_H__
#define __PMU_DEVICE_H__

#include <stdint.h>
#include "hdf.h"

/**
 * @brief The enumeration of wakeup time.
 */
typedef enum {
    PMU_WAKEUP_TIME_100,
    PMU_WAKEUP_TIME_200,
    PMU_WAKEUP_TIME_500,
    PMU_WAKEUP_TIME_1000,
    PMU_WAKEUP_TIME_2000,
    PMU_WAKEUP_TIME_4000,
    PMU_WAKEUP_TIME_6000,
    PMU_WAKEUP_TIME_8000,
    PMU_WAKEUP_TIME_10000,
    PMU_WAKEUP_TIME_12000,
    PMU_WAKEUP_TIME_MAX = PMU_WAKEUP_TIME_12000,   
} PMU_WAKEUP_TIME_t;

/**
 * @brief The enumeration of brownout mode.
 */
typedef enum {
    PMU_BROWNOUT_NONE,
    PMU_BROWNOUT_RESET,
    PMU_BROWNOUT_INTERRUPT,
} PMU_BROWNOUT_MODE_t;

/**
 * @brief The data structure of wakeup timer settings.
 */
struct PMU_WakeupTime {
    uint8_t manissa;
    uint8_t exp;
};

/**
 * @brief The wakeup timer settings for each enumeration of wakeup time.
 */
static const struct PMU_WakeupTime PMU_WAKEUP_TIME_tbl[PMU_WAKEUP_TIME_MAX+1] =
{
    [PMU_WAKEUP_TIME_100] = {.manissa = 4, .exp = 7},
    [PMU_WAKEUP_TIME_200] = {.manissa = 4, .exp = 8},
    [PMU_WAKEUP_TIME_500] = {.manissa = 10, .exp = 8},
    [PMU_WAKEUP_TIME_1000] = {.manissa = 10, .exp = 9},
    [PMU_WAKEUP_TIME_2000] = {.manissa = 10, .exp = 10},
    [PMU_WAKEUP_TIME_4000] = {.manissa = 10, .exp = 11},
    [PMU_WAKEUP_TIME_6000] = {.manissa = 15, .exp = 11},
    [PMU_WAKEUP_TIME_8000] = {.manissa = 10, .exp = 12},
    [PMU_WAKEUP_TIME_10000] = {.manissa = 12, .exp = 12},
    [PMU_WAKEUP_TIME_12000] = {.manissa = 15, .exp = 12},   
};

/**
 * @brief Sleep in Low power mode then Continue.
 *
 * @param time The enumeration of sleep time.
 * @return None
 * This function puts the system in low power mode and then sleep for the time defined, 
 * then wake up and continue to run. This function can only be used for the hardware
 * which is configured to support low power mode.
 * Need to be very cautious to use this function to avoid of entering a deep
 * sleep mode unrecoverable.
 */
void PMU_devSleepLowPwr(PMU_WAKEUP_TIME_t time);

/**
 * @brief Sleep in Low power mode and Reboot
 *
 * @param time The enumeration of sleep time
 * @return None
 * This function puts the system in low power mode and then sleep for the time defined, 
 * then reboot when the timer expires. This function can only be used for the hardware
 * which is configured to support low power mode.
 * Need to be very cautious to use this function to avoid of entering a deep
 * sleep mode unrecoverable.
 */
void PMU_devSleepLowPwrReboot(PMU_WAKEUP_TIME_t time);

/**
 * @brief Initialize brownout settings.
 *
 * @param brown The enumeration of brownout mode.
 * @param volt The brownout voltage.
 * @param cb The pointer to the call back function for brown interrupt.
 */
void PMU_devBrownOutInit(PMU_BROWNOUT_MODE_t brown, uint32_t volt, void *cb);

/**
 * @brief Set system power to battery mode.
 */
void PMU_devSetPowerModeBattery(void);

/**
 * @brief Set system power to LDO mode.
 */
void PMU_devSetPowerModeLDO(void);

#endif /* __PMU_DEVICE_H__ */
