/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file rtc_device.h
 */

#ifndef __RTC_DEVICE_H__
#define __RTC_DEVICE_H__

#include <stdint.h>
#include "hdf.h"

typedef enum {
    RTC_ALARM_0,
    RTC_ALARM_1,
    RTC_ALARM_MAX,   
} RTC_ALARM_t;

typedef enum {
    RTC_ALARM_MODE_IDLE,   
    RTC_ALARM_MODE_SINGLE,
    RTC_ALARM_MODE_REPEAT,
} RTC_ALARM_MODE_t;

/**
 * @brief A structure to represent RTC device
 */
typedef struct {
    IRQn_Type IRQ;
    int32_t (*RTC_Init)(void);
    int32_t (*RTC_Stop)(void);
    int32_t (*Alarm_Start)(RTC_ALARM_t alarm, RTC_ALARM_MODE_t mode, uint32_t sec, void *cb);
    int32_t (*Alarm_Stop)(RTC_ALARM_t alarm);
} RTC_Device_t;

typedef void (*rtc_cb_func_t)(RTC_Device_t *dev);

/**
 * @brief Request A RTC Device
 *
 * @return The pointer to the RTC device instance.
 */
RTC_Device_t *RTC_devRequest(void);

/**
 * @brief Initialize RTC Timer
 *
 * @param dev The pointer to RTC device,
 * @return 0 for success or error code upon a failure.
 */
int32_t RTC_devInit(RTC_Device_t *dev);

/**
 * @brief Stop RTC Timer
 *
 * @param dev The pointer to RTC device,
 * @return 0 for success or error code upon a failure.
 */
int32_t RTC_devStop(RTC_Device_t *dev);

/**
 * @brief Start a Alarm Timer
 *
 * @param dev The pointer to RTC device,
 * @param alarm The enumeration of the alarm to be started.
 * @param mode The enumeration of alarm mode.
 * @param sec The seconds for the alarm to fire.
 * @param cb The pointer to the call back function when the alarm fires.
 * @return 0 for success or error code upon a failure.
 */
int32_t RTC_devStartAlarm(RTC_Device_t *dev, RTC_ALARM_t alarm,
                         RTC_ALARM_MODE_t mode, uint32_t sec, void *cb);

/**
 * @brief Stop a Alarm Timer
 *
 * @param dev The pointer to RTC device,
 * @param alarm The enumeration of the alarm to be started.
 * @return 0 for success or error code upon a failure.
 */
int32_t RTC_devStopAlarm(RTC_Device_t *dev, RTC_ALARM_t alarm);

#endif /* __RTC_DEVICE_H__ */