/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file rx_device.h
 */

#ifndef __RX_DEVICE_H__
#define __RX_DEVICE_H__

#include "hdf.h"
#include "rf_codec.h"

typedef const struct RX_Device_t RX_Device_Type;
typedef void (*rx_cb_func_t)(RX_Device_Type *dev);

/**
 * @brief A structure to represent RX device state.
 */
typedef struct {
    DCM_FMT_t format;
    uint8_t message_frame_size;
    uint8_t buffer_size;
    uint8_t buffer[RX_BUFF_SIZE];
    rx_cb_func_t cb;
} RX_Dev_State_t;

/**
 * @brief A structure to represent RX device driver.
 */
typedef const struct RX_Device_t {
    int32_t (*Init)(uint8_t bytes, DCM_FMT_t fmt);
    int32_t (*Stop)(void);
    int32_t (*GetReceivedData)(uint8_t *data, uint8_t size);
    int32_t (*RegisterMsgIRQ)(void *cb);
    int32_t (*UnregisterMsgIRQ)(void);
    char *Name;
    IRQn_Type IRQ;
    RX_Dev_State_t *state;
} RX_Device_t;

/**
 * @brief Request a RX device.
 *
 * @return The pointer to the RX device or NULL upon a failure.
 */
RX_Device_t *RX_devRequest(void);

/**
 * @brief Initialize RX device
 *
 * @param dev Pointer to RX Device structure
 * @param bytes The number of bytes expect to receive, the message length.
 * @param fmt The enum for the duty-cycle modulation format.
 * @return 0 for success or error code upon a failure
 */
int32_t RX_devInit(RX_Device_t *dev, uint8_t bytes, DCM_FMT_t fmt);

/**
 * @brief Send data to RX device.
 *
 * @param dev Pointer to RX Device structure.
 * @param data Pointer to the data memory to save the received data.
 * @param len The size of the data memory.
 * @return 0 for success or error code upon a failure
 */
int32_t RX_devGetReceivedData(RX_Device_t *dev, uint8_t *data, uint32_t len);

/**
 * @brief Stop Receiver.
 *
 * @param dev Pointer to RX Device structure.
 * @return 0 for success or error code upon a failure
 */
int32_t RX_devStop(RX_Device_t *dev);

/**
 * @brief Register a call back function to RX device driver
 *
 * @param dev Pointer to RX Device structure
 * @param cb The pointer to the call back function.
 * @return 0 for success or error code upon a failure
 */
int32_t RX_devRegisterCallback(RX_Device_t *dev, void *cb);

/**
 * @brief Un-Register a call back function to RX device driver
 *
 * @param dev Pointer to RX Device structure
 * @return 0 for success or error code upon a failure
 */
int32_t RX_devUnregisterCallback(RX_Device_t *dev);

#endif /* __RX_DEVICE_H__ */
