/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file spi_device.h
 */

#ifndef __SPI_DEVICE_H__
#define __SPI_DEVICE_H__

#include "hdf.h"
#include "errno.h"

#define SPI_BUFF_SIZE 16

/**
 * @brief The enumeration of SPI phase
 */
enum SPI_DEV_CLOCK_PHASE_t {
    SPI_DEV_CLOCK_PHASE0,
    SPI_DEV_CLOCK_PHASE1,
};

/**
 * @brief The enumeration of DPI polarity
 */
enum SPI_DEV_CLOCK_POLARITY_t {
    SPI_DEV_CLOCK_POLARITY0,
    SPI_DEV_CLOCK_POLARITY1,
};

/**
 * @brief The enumeration of SPI mode
 */
enum SPI_DEV_MODE_t {
    SPI_DEV_MODE_SLAVE,
    SPI_DEV_MODE_MASTER,
};

/**
 * @brief The enumeration of clock divider for SPI device
 */
enum {
    SPI_DEV_CLOCK_DIV2,
    SPI_DEV_CLOCK_DIV4,
    SPI_DEV_CLOCK_DIV8,
    SPI_DEV_CLOCK_DIV16,
    SPI_DEV_CLOCK_DIV32,
    SPI_DEV_CLOCK_DIV64,
    SPI_DEV_CLOCK_DIV128,
    SPI_DEV_CLOCK_DIV256,
    SPI_DEV_CLOCK_DIV512,
    SPI_DEV_CLOCK_DIV1024,
    SPI_DEV_CLOCK_DIV2048,
    SPI_DEV_CLOCK_DIV4096,
};

/**
 * @brief A structure to represent SPI device initialization data.
 */  
typedef struct {
    uint8_t Mode;
    uint8_t ClockDiv;
    uint8_t Phase;
    uint8_t Polarity;
    uint8_t IntDlyCnt;
} SPI_Device_init_t;

/**
 * @brief A structure to represent SPI device
 */ 
typedef const struct {
    void (*Enable)(void);
    void (*Disable)(void);
    void (*IrqEnable)(void);
    void (*IrqDisable)(void);
    uint8_t (*ReadData)(void);
    uint8_t (*GetStatus)(void);
    void (*WriteData)(uint8_t data);
    void (*SetMode)(uint8_t mode);
    void (*SetClockDivider)(uint8_t div);
    void (*SetPhase)(uint8_t pha);
    void (*SetPolarity)(uint8_t pol);
    void (*IRQDelay)(uint8_t cnt);
    void (*ClearIRQ)(void);
    IRQn_Type IRQ;
    char *Name;
} SPI_Device_t;

/**
 * @brief Set Clock Divider for SPI Device
 *
 * @param div The enumeration of the divider.
 * @return None.
 */
void SPI_DevSetClockDivider(uint8_t div);

/**
 * @brief Request a SPI Device.
 *
 * @param name The name string of SPI device.
 * @return The pointer to the SPI device instance.
 */
SPI_Device_t *SPI_devRequest(const char *name);

/**
 * @brief Initialize SPI device.
 *
 * @param dev The pointer to SPI device
 * @param init Pointer to the initialization data struct.
 * @return 0 for success or error code upon a failure.
 */
int32_t SPI_devInit(SPI_Device_t *dev, SPI_Device_init_t *init);

/**
 * @brief Read Data from SPI device.
 *
 * @param dev The pointer to SPI device
 * @param cmd The enumeration of SPI command.
 * @param data Pointer to the returned data block from SPI device.
 * @param len The length of the data block to be returned from SPI device.
 * @return 0 for success or error code upon a failure.
 */
int32_t SPI_devReadData(SPI_Device_t *dev, uint8_t cmd, uint8_t *data, uint32_t len);

/**
 * @brief Write Data to SPI device.
 *
 * @param dev The pointer to SPI device
 * @param cmd The enumeration of SPI command.
 * @param data Pointer to the data block to be sent to SPI device.
 * @param len The length of the data block to be sent to SPI device.
 * @return 0 for success or error code upon a failure.
 */
int32_t SPI_devWriteData(SPI_Device_t *dev, uint8_t cmd, uint8_t *data, uint32_t len);

#endif /* __SPI_DEVICE_H__ */