/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file systick_device.h
 */

#ifndef ___SYSTICK_DEVICE_H__
#define ___SYSTICK_DEVICE_H__

#include <stdint.h>
#include "hdf.h"
#include "errno.h"

enum {
    SYSTICK_MODE_DISABLE,
    SYSTICK_MODE_ENABLE,
    SYSTICK_MODE_ENABLE_INTERRUPT,
};    

/**
 * @brief A structure to represent WDT device
 */
typedef const struct {
    void (*Enable)(void);
    void (*Disable)(void);
    uint8_t (*IsEnabled)(void);
    void (*IntEnable)(void);
    void (*IntDisable)(void);
    void (*SetLoad)(uint32_t);
    uint32_t (*GetLoad)(void);
    void (*SetCount)(uint32_t);
    uint32_t (*GetCount)(void);
    IRQn_Type IRQ;
} SysTick_Device_t;

/**
 * @brief The SysTick timer interrupt call back function.
 */
typedef void (*systick_cb_func_t)(SysTick_Device_t *dev);


/**
 * @brief Request a SysTick Device
 *
 * @return The pointer to the timer device instance.
 */
SysTick_Device_t *SysTick_devRequest(void);

/**
 * @brief SysTick time delay in tick
 *
 * @param dev The pointer to SysTick device,
 * @param ticks The time delay in ticks.
 * @return 0 for success or error code upon a failure.
 */
int32_t SysTick_devDelayTicks(SysTick_Device_t *dev, uint32_t ticks);

/**
 * @brief SysTick time delay in ms
 *
 * @param dev The pointer to SysTick device,
 * @param ms The time delay in ms.
 * @return 0 for success or error code upon a failure.
 */
int32_t SysTick_devDelayMs(SysTick_Device_t *dev, uint32_t ms);

/**
 * @brief Get Systick interrupt counts.
 *
 * @param dev The pointer to SysTick device,
 * @return The interrupt counts for success or error code upon a failure.
 */
int32_t SysTick_devGetInterruptCounts(SysTick_Device_t *dev);

/**
 * @brief Get elapsed time in ms since last systick initialization.
 *
 * @param dev The pointer to SysTick device,
 * @return The elapsed time in ms for success or error code upon a failure.
 */
int64_t SysTick_devGetElapsedMs(SysTick_Device_t *dev);

/**
 * @brief Initialize SysTick Timer
 *
 * @param dev The pointer to SysTick device,
 * @param mode The enumeration of SysTick timer mode.
 * @param ticks The reload counter settings.
 * @param cb The pointer to the call back function if it is set to interrupt mode.
 * @return 0 for success or error code upon a failure.
 */
int32_t SysTick_devInit(SysTick_Device_t *dev, uint8_t mode, uint32_t ticks, void *cb);

#endif /* ___SYSTICK_DEVICE_H__ */

