/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file timer_device.h
 */

#ifndef __TIMER_DEVICE_H__
#define __TIMER_DEVICE_H__

#include <stdint.h>
#include "hdf.h"
#include "errno.h"

typedef const struct Timer_Device_t Timer_Device_Type;
typedef void (*timer_cb_func_t)(Timer_Device_Type *dev);

/**
 * @brief A structure to represent timer device state
 */
typedef struct {
    timer_cb_func_t cb_func;
    uint32_t    tick;
} Timer_State_t;

/**
 * @brief A structure to represent timer device
 */
typedef const struct Timer_Device_t {
    void (*Start)(void);
    void (*Stop)(void);
    void (*LoadCounter)(uint32_t cnt);
    IRQn_Type IRQ;
    uint8_t Num;
    Timer_State_t *State;
    char *Name;
    void *SFRS;
}Timer_Device_t;

/**
 * @brief Start a timer
 *
 * @param dev The pointer to the timer device.
 * @return 0 for success or error code upon a failure.
 */
int32_t Timer_devStart(Timer_Device_t *dev);

/**
 * @brief Stop a timer
 *
 * @param dev The pointer to the timer device.
 * @return 0 for success or error code upon a failure.
 */
int32_t Timer_devStop(Timer_Device_t *dev);

/**
 * @brief Set timing interval of a timer
 *
 * @param dev The pointer to the timer device.
 * @param ms The interval to reload the timer.
 * @return 0 for success or error code upon a failure.
 */
int32_t Timer_devSetMs(Timer_Device_t *dev, uint32_t ms);

/**
 * @brief Register interrupt handling function to a timer device.
 *
 * @param dev The pointer to the timer device.
 * @param cb Pointer to the call back function to handle the timer interrupt.
 * @return 0 for success or error code upon a failure.
 */
int32_t Timer_devRegisterIRQ(Timer_Device_t *dev, void *cb);

/**
 * @brief Unregister interrupt handling function to a timer device.
 *
 * @param dev The pointer to the timer device.
 * @param cb Pointer to the call back function to handle the timer interrupt.
 * @return 0 for success or error code upon a failure.
 */
int32_t Timer_devUnregisterIRQ(Timer_Device_t *dev, void *cb);

/**
 * @brief Enable the interrupt of a timer device.
 *
 * @param dev The pointer to the timer device.
 * @return 0 for success or error code upon a failure.
 */
int32_t Timer_devEnableIRQ(Timer_Device_t *dev);

/**
 * @brief Disable the interrupt of a timer device.
 *
 * @param dev The pointer to the timer device.
 * @return 0 for success or error code upon a failure.
 */
int32_t Timer_devDisableIRQ(Timer_Device_t *dev);

/**
 * @brief Clear the tick counter of a timer interrupt.
 *
 * @param dev The pointer to the timer device.
 * @return 0 for success or error code upon a failure.
 */
int32_t Timer_devClearTick(Timer_Device_t *dev);

/**
 * @brief Get the tick counter of a timer device.
 *
 * @param dev The pointer to the timer device.
 * @return the counter of timer interrupt or error code upon a failure.
 */
int32_t Timer_devGetTick(Timer_Device_t *dev);

/**
 * @brief Request a Timer Device
 *
 * @param name The name string of timer device.
 * @return The pointer to the timer device instance.
 */
Timer_Device_t *Timer_devRequest(const char *name);

#endif /* __TIMER_DEVICE_H__ */