/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file tx_device.h
 */

#ifndef __TX_DEVICE_H__
#define __TX_DEVICE_H__

#include <stdint.h>
#include "hdf.h"
#include "rf_codec.h"

#define TX_BUFF_SIZE 128

typedef struct {
    DCM_FMT_t format;
    uint16_t burst_size;
    uint8_t wait_size;
}TX_Config_t;

/**
 * @brief A structure to represent TX device.
 */
typedef struct {
    int32_t (*SendData)(uint8_t *data, uint32_t len);
    int32_t (*Stop)(void);
    int32_t (*Init)(TX_Config_t *cfg);
    int32_t (*RegisterCallback)(void *cb);
    int32_t (*UnregisterCallback)(void);
    int32_t (*IsBusy)(void);
    IRQn_Type IRQ_Reload;
    IRQn_Type IRQ_Done;
    char *Name;
} TX_Device_t;

typedef void (*tx_cb_func_t)(TX_Device_t *dev);

/**
 * @brief Initialize TX device
 *
 * @param dev Pointer to TX Device structure
 * @param cfg Pointer to TX config data structure
 * @return 0 for success or error code upon a failure
 */
int32_t TX_devInit(TX_Device_t *dev, TX_Config_t *cfg);

/**
 * @brief Request a TX device.
 *
 * @return The pointer to the TX device or NULL upon a failure.
 */
TX_Device_t *TX_devRequest(void);

/**
 * @brief Check if TX device busy.
 *
 * @param dev Pointer to TX Device structure.
 * @return the number of bytes pending on TX or error code upon a failure
 */
int32_t TX_devIsBusy(TX_Device_t *dev);

/**
 * @brief Send data to TX device.
 *
 * @param dev Pointer to TX Device structure.
 * @param data Pointer to the data memory to be sent.
 * @param len The size of the data memory.
 * @return 0 for success or error code upon a failure
 */
int32_t TX_devSendData(TX_Device_t *dev, uint8_t *data, uint32_t len);

/**
 * @brief Stop transmission.
 *
 * @param dev Pointer to TX Device structure.
 * @return 0 for success or error code upon a failure
 */
int32_t TX_devStop(TX_Device_t *dev);

/**
 * @brief Register a call back function to TX device driver
 *
 * @param dev Pointer to TX Device structure
 * @param cb The pointer to the call back function.
 * @return 0 for success or error code upon a failure
 */
int32_t TX_devRegisterCallback(TX_Device_t *dev, void *cb);

/**
 * @brief Un-Register a call back function to TX device driver
 *
 * @param dev Pointer to TX Device structure
 * @return 0 for success or error code upon a failure
 */
int32_t TX_devUnregisterCallback(TX_Device_t *dev);

#endif /* __TX_DEVICE_H__ */
