/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file pir_device.c
 */

#include <stdint.h>
#include "hdf.h"
#include "pir_device.h"
#include "errno.h"

#define CLOCK           10000   /* 10KHz */
#define INHB_FACTOR     2048


static int32_t pir_init(uint8_t clkdiv, uint8_t etime, uint8_t itime, uint8_t atten);
static int32_t pir_enable_wakeup(void *cb);
static int32_t pir_disable_wakeup(void);
static int32_t pir_disable(void);

/**
 * @brief The instance of RTC device
 */
#ifdef PIR_SFRS
const static PIR_Device_t PIR_Device =
{
    .IRQ = PIR_EVENT_IRQn,
    .Init = pir_init,
    .EnableWakeup = pir_enable_wakeup,
    .DisableWakeup = pir_disable_wakeup,
    .Disable = pir_disable,
};    
#endif

PIR_Device_t *PIR_devRequest(void)
{
#ifdef PIR_SFRS
    return (PIR_Device_t *)&PIR_Device;
#else
    return NULL,
#endif
}

typedef struct {
    pir_cb_func_t cb_func;
    uint8_t wakeup_flag;
} PIR_State_t;

PIR_State_t PIR_State = {
    .cb_func = NULL,
    .wakeup_flag = 0,
};

static int32_t pir_init(uint8_t clkdiv, uint8_t etime, uint8_t itime, uint8_t atten)
{
    uint32_t inhibit;
    uint32_t counter;
    uint8_t divider;
   
    counter = (CLOCK * etime + (clkdiv * 10)/2);
    counter = counter/(clkdiv * 10);
    inhibit = (CLOCK * itime + (clkdiv * INHB_FACTOR / 2))/(clkdiv * INHB_FACTOR);
    divider = clkdiv/E_PIR_DIVIDER_STEPSIZE - 1;
    
    if PIR_INHB_NOT_IN_RANGE(inhibit)
        return -EPERM;
    if PIR_DIVIDER_NOT_IN_RANGE(divider)
        return -EPERM;
    if PIR_COUNTER_NOT_IN_RANGE(counter)
        return -EPERM;
    if PIR_ATTEN_NOT_IN_RANGE(atten / E_PIR_ATTEN_STEPSIZE)
        return -EPERM;
    PIR_SetDivider(divider);
    PIR_SetInhibit(inhibit);
    PIR_SetAtten(atten / E_PIR_ATTEN_STEPSIZE);
    PIR_SetCounter(counter);
    PIR_Enable();
    
    return 0;
}

static int32_t pir_enable_wakeup(void *cb)
{
    PIR_State.wakeup_flag = 0;
    PIR_State.cb_func = (pir_cb_func_t)cb;
    NVIC_ClearPendingIRQ(PIR_EVENT_IRQn);
    NVIC_EnableIRQ(PIR_EVENT_IRQn);
    PIR_Enable();
    return 0;
}

static int32_t pir_disable_wakeup(void)
{
    NVIC_DisableIRQ(PIR_EVENT_IRQn);
    PIR_Disable();
    return 0;
}

static int32_t pir_disable(void)
{
    PIR_Disable();
    return 0;
}

void PIR_Handler(void)
{
    PIR_State.wakeup_flag = 1;
    if (PIR_State.cb_func)
        PIR_State.cb_func((PIR_Device_t *)&PIR_Device);
}

int32_t PIR_devInit(PIR_Device_t * dev, uint8_t clkdiv, uint8_t etime, uint8_t itime, uint8_t atten)
{
    if (!dev)
        return -EFAULT;
    
    return pir_init(clkdiv, etime, itime, atten);
}


int32_t PIR_devEnableWakeup(PIR_Device_t *dev, void *cb)
{
    if (!dev)
        return -EFAULT;
 
    return pir_enable_wakeup(cb);
}

int32_t PIR_devDisableWakeup(PIR_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    
    return pir_disable_wakeup();
}


int32_t PIR_devDisable(PIR_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    
    return pir_disable();
}