/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file rf_codec.c
 */

#include <string.h>
#include "rf_codec.h"
#include "errno.h"

#define DCM_3BIT_MASK           0x07
#define DCM_3BIT_LENGTH         0x03
#define DCM_3BIT_ONE            0x04
#define DCM_3BIT_ZERO           0x06

#define DCM_4BIT_MASK           0x0F
#define DCM_4BIT_LENGTH         0x04
#define DCM_4BIT_ONE            0x08
#define DCM_4BIT_ZERO           0x0E

#define DCM_5BIT_MASK           0x1F
#define DCM_5BIT_LENGTH         0x05
#define DCM_5BIT_ONE            0x10
#define DCM_5BIT_ZERO           0x3E

static const uint16_t dcm_4bit_lut[16] = {
    [0] = 0xEEEE,
    [1] = 0xEEE8,
    [2] = 0xEE8E,
    [3] = 0xEE88,
    [4] = 0xE8EE,
    [5] = 0xE8E8,
    [6] = 0xE88E,
    [7] = 0xE888,
    [8] = 0x8EEE,
    [9] = 0x8EE8,
    [10] = 0x8E8E,
    [11] = 0x8E88,
    [12] = 0x88EE,
    [13] = 0x88E8,
    [14] = 0x888E,
    [15] = 0x8888,   
};


#define MAX_TX_BURST_BIT_SIZE           1024
#define MAX_TX_PREAMBLE_BIT_SIZE        16
#define MAX_TX_SYNC_BIT_SIZE            16
#define MAX_TX_CRC_BIT_SIZE             8
#define MAX_TX_MAX_DATA_BIT_SIZE        (MAX_TX_BURST_BIT_SIZE \
                                         - MAX_TX_PREAMBLE_BIT_SIZE \
                                         - MAX_TX_SYNC_BIT_SIZE \
                                         - MAX_TX_CRC_BIT_SIZE)
#define MAX_TX_BYTE_SIZE                (MAX_TX_MAX_DATA_BIT_SIZE / 8);            



// The chips in the template are 1 0 0 for all eight bits.  To use the template,
// use bitwise or operator to change the middle chip to a 1 for all high duty
// cycle symbols.  We dont use the upper byte
// Content => 1 0 0 1  |  0 0 1 0  |  0 1 0 0  |  1 0 0 1  |  0 0 1 0  |  0 1 0 0
#define ASK3DCM_PAYLOAD_BYTE_TEMPLATE 0x00924924UL


// Look up table to populate information carrying "middle" chips for a nibble's worth
// of payload.  We use the nibble as an index for this table and the table value is
// bitwise-ored with the template to create the data pattern.  For the upper nibble, 
// we shift the nibble right by 4 to create the index and we extend the table calue 
// into a 32 bit word and shift the output left by 12 bits before oring with the
// template.
// future improvement suggestion - implement full 256 byte-index LUT
static const uint16_t ASK3DCM_PAYLOAD_NIBBLE_LUT[16] = {
  0x0492, // 0 => x 0 x x    0 x x 0     x x 0 x     x 1 x x      1 x x 1    x x 1 x
  0x0490, // 1 => x 0 x x    0 x x 0     x x 0 x     x 1 x x      1 x x 1    x x 0 x
  0x0482, // 2 => x 0 x x    0 x x 0     x x 0 x     x 1 x x      1 x x 0    x x 1 x
  0x0480, // 3 => x 0 x x    0 x x 0     x x 0 x     x 1 x x      1 x x 0    x x 0 x
  0x0412, // 4 => x 0 x x    0 x x 0     x x 0 x     x 1 x x      0 x x 1    x x 1 x
  0x0410, // 5 => x 0 x x    0 x x 0     x x 0 x     x 1 x x      0 x x 1    x x 0 x
  0x0402, // 6 => x 0 x x    0 x x 0     x x 0 x     x 1 x x      0 x x 0    x x 1 x
  0x0400, // 7 => x 0 x x    0 x x 0     x x 0 x     x 1 x x      0 x x 0    x x 0 x
  0x0092, // 8 => x 0 x x    0 x x 0     x x 0 x     x 0 x x      1 x x 1    x x 1 x
  0x0090, // 9 => x 0 x x    0 x x 0     x x 0 x     x 0 x x      1 x x 1    x x 0 x
  0x0082, // A => x 0 x x    0 x x 0     x x 0 x     x 0 x x      1 x x 0    x x 1 x
  0x0080, // B => x 0 x x    0 x x 0     x x 0 x     x 0 x x      1 x x 0    x x 0 x
  0x0012, // C => x 0 x x    0 x x 0     x x 0 x     x 0 x x      0 x x 1    x x 1 x
  0x0010, // D => x 0 x x    0 x x 0     x x 0 x     x 0 x x      0 x x 1    x x 0 x
  0x0002, // E => x 0 x x    0 x x 0     x x 0 x     x 0 x x      0 x x 0    x x 1 x
  0x0000  // F => x 0 x x    0 x x 0     x x 0 x     x 0 x x      0 x x 0    x x 0 x
};


// macro for encoding a byte of data into ASK {2/3 , 1/3} duty cycle modulation
// future improvement suggestion - implement full 256 byte-index LUT
static uint32_t EncodeASK3DCMByte( const uint8_t pay )
{
    return (ASK3DCM_PAYLOAD_NIBBLE_LUT[pay & 0x0F]                               |   // low  nibble data chips
            (((uint32_t) ASK3DCM_PAYLOAD_NIBBLE_LUT[(pay & 0xF0) >> 4])  << 12)  |   // high nibble data chips
            ASK3DCM_PAYLOAD_BYTE_TEMPLATE);                                             // template non-data chips  
}

int32_t DCM_Encoder(DCM_FMT_t mode, uint8_t *src, uint8_t *des, uint8_t len)
{
    int32_t j = 0, ret;
    union {
        uint32_t word;
        uint16_t hword[2];
        uint8_t byte[4];
    } tempw;
    uint8_t data;
    
    switch (mode) {
    case DCM_FMT_3BIT:
        while( j < len) {
            /* expand the byte into chips */
            tempw.word = EncodeASK3DCMByte(*src);
            *(des++) = tempw.byte[2];
            *(des++) = tempw.byte[1];
            *(des++) = tempw.byte[0];
            src++;
            j++;
        }
        ret = len * DCM_3BIT_LENGTH;
        break;
    case DCM_FMT_4BIT:
        while( j < len) {
            data = *src;
            tempw.hword[0] = dcm_4bit_lut[(data >> DCM_4BIT_LENGTH) & DCM_4BIT_MASK];
            tempw.hword[1] = dcm_4bit_lut[data & DCM_4BIT_MASK];
            *(des++) = tempw.byte[1];
            *(des++) = tempw.byte[0];
            *(des++) = tempw.byte[3];
            *(des++) = tempw.byte[2];
            j++;
            src++;
        }
        ret = len * DCM_4BIT_LENGTH;
        break;
    default:
        break;
    }
    return ret;
}
