/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file rtc_device.c
 */

#include "rtc_device.h"
#include "errno.h"

static int32_t rtc_init(void);
static int32_t rtc_stop(void);
static int32_t alarm_start(RTC_ALARM_t alarm,
                         RTC_ALARM_MODE_t mode, uint32_t secs, void *cb);
static int32_t alarm_stop(RTC_ALARM_t alarm);
static void alarm_update(RTC_ALARM_t alarm);

/**
 * @brief The instance of RTC device
 */
#ifdef RTC_SFRS
const static RTC_Device_t RTC_Device =
{
    .IRQ = RTC_OVFL_IRQn,
    .RTC_Init = rtc_init,
    .RTC_Stop = rtc_stop,
    .Alarm_Start = alarm_start,
    .Alarm_Stop = alarm_stop,
};    
#endif

RTC_Device_t *RTC_devRequest(void)
{
#ifdef RTC_SFRS
    return (RTC_Device_t *)&RTC_Device;
#else
    return NULL,
#endif
}

typedef struct {
    IRQn_Type IRQ;
    RTC_ALARM_MODE_t mode;
    rtc_cb_func_t cb_func;
    uint8_t active;
    uint32_t rtc_secs;
    uint32_t alarm_secs;
    uint32_t (*get_counter)(void);
    void (*set_counter)(uint32_t cnt);
} Alarm_State_t;
    
/**
 * @brief A structure to represent RTC device state
 */
typedef struct {
    Alarm_State_t alarm[RTC_ALARM_MAX];
    uint32_t    tick_overflow;
} RTC_State_t;

RTC_State_t RTC_State = {
    .alarm[RTC_ALARM_0] = {
        .set_counter = f_RTC_SetAlarm0Counter,
        .get_counter = f_RTC_GetAlarm0Counter,
        .IRQ = ALARM0_IRQn
    },
    .alarm[RTC_ALARM_1] = {
        .set_counter = f_RTC_SetAlarm1Counter,
        .get_counter = f_RTC_GetAlarm1Counter,
        .IRQ = ALARM1_IRQn
    },
    .tick_overflow = 0,
};
static int32_t rtc_init(void)
{
    f_CLOCK_EnableXTClock32K();

    NVIC_ClearPendingIRQ(RTC_Device.IRQ);
    NVIC_EnableIRQ(RTC_Device.IRQ);
    f_RTC_EnableOverflowIRQ();
    f_RTC_DisableAlarmIRQ();
    RTC_State.tick_overflow = 0;
    
    f_RTC_Unlock();
    f_RTC_Enable();
    f_RTC_SetCurrentCounter(0);
    f_RTC_Lock();
    return 0;
}

static int32_t rtc_stop(void)
{
    f_CLOCK_DisableXTClock32K();
    
    f_RTC_Unlock();
    NVIC_DisableIRQ(RTC_Device.IRQ);
    f_RTC_EnableOverflowIRQ();
    f_RTC_DisableAlarmIRQ();
    RTC_State.tick_overflow = 0;
    f_RTC_Disable();
    f_RTC_Lock();
    return 0;
}

static int32_t alarm_start(RTC_ALARM_t alarm,
                         RTC_ALARM_MODE_t mode, uint32_t secs, void *cb)
{
    if (alarm >= RTC_ALARM_MAX)
        return -ENODEV;

    NVIC_ClearPendingIRQ(RTC_State.alarm[alarm].IRQ);
    NVIC_EnableIRQ(RTC_State.alarm[alarm].IRQ);
    RTC_State.alarm[alarm].cb_func = (rtc_cb_func_t)cb;
    RTC_State.alarm[alarm].mode = mode;
    RTC_State.alarm[alarm].active = 1;
    RTC_State.alarm[alarm].alarm_secs = secs;
    alarm_update(alarm);
    return 0;
}

static int32_t alarm_stop(RTC_ALARM_t alarm)
{
    if (alarm >= RTC_ALARM_MAX)
        return -ENODEV;

    NVIC_DisableIRQ(RTC_State.alarm[alarm].IRQ);
    RTC_State.alarm[alarm].active = 0;
    RTC_State.alarm[alarm].alarm_secs = 0;
    alarm_update(alarm);
    return 0;
}

static void alarm_handler(RTC_ALARM_t alarm)
{
    uint32_t c_cnt, a_cnt;
    uint8_t i;

    f_RTC_Unlock();
    for (i = 0; i < RTC_ALARM_MAX; i++) {
        if (i == alarm) {
           if (RTC_State.alarm[i].mode == RTC_ALARM_MODE_REPEAT) {
                RTC_State.alarm[i].set_counter(RTC_State.alarm[i].alarm_secs);
                RTC_State.alarm[i].active = 1;
           } else {
                NVIC_DisableIRQ(RTC_State.alarm[i].IRQ);
                RTC_State.alarm[i].active = 0;
                RTC_State.alarm[i].alarm_secs = 0;
                RTC_State.alarm[i].set_counter(0);
           }
        } else {
            if ((RTC_State.alarm[i].mode != RTC_ALARM_MODE_IDLE) &&
                (RTC_State.alarm[i].active != 0)) {
                a_cnt = RTC_State.alarm[i].get_counter();
                c_cnt = f_RTC_GetCurrentCounter();
                if (a_cnt > c_cnt)
                    RTC_State.alarm[i].set_counter(a_cnt - c_cnt);
            }
        }
    }
    f_RTC_SetCurrentCounter(0);
    f_RTC_Lock();
    if (RTC_State.alarm[alarm].cb_func != NULL)
        RTC_State.alarm[alarm].cb_func((RTC_Device_t *)&RTC_Device);
}

static void alarm_update(RTC_ALARM_t alarm)
{
    uint32_t c_cnt, a_cnt;
    uint8_t i, irqs = 0;

    f_RTC_Unlock();
    for (i = 0; i < RTC_ALARM_MAX; i++) {
        if (i == alarm) {
            if (RTC_State.alarm[i].mode != RTC_ALARM_MODE_IDLE) {
                RTC_State.alarm[i].set_counter(RTC_State.alarm[i].alarm_secs);
                RTC_State.alarm[i].active = 1;
                irqs++;
           } else {
                RTC_State.alarm[i].active = 0;
                RTC_State.alarm[i].alarm_secs = 0;
                RTC_State.alarm[i].set_counter(0);
           }
        } else {
            if ((RTC_State.alarm[i].mode != RTC_ALARM_MODE_IDLE) &&
                (RTC_State.alarm[i].active != 0)) {
                a_cnt = RTC_State.alarm[i].get_counter();
                c_cnt = f_RTC_GetCurrentCounter();
                irqs++;
                if (a_cnt > c_cnt)
                    RTC_State.alarm[i].set_counter(a_cnt - c_cnt);
            }
        }
    }
    f_RTC_SetCurrentCounter(0);
    if (irqs != 0)
        f_RTC_EnableAlarmIRQ();
    else
        f_RTC_DisableAlarmIRQ();
    f_RTC_Lock();
}


void ALARM0_Handler(void)
{
    RTC_State.alarm[RTC_ALARM_0].active = 0;
    alarm_handler(RTC_ALARM_0);
}

void ALARM1_Handler(void)
{
    RTC_State.alarm[RTC_ALARM_1].active = 0;
    alarm_handler(RTC_ALARM_1);
}

void RTC_OVFL_Handler(void)
{
}

int32_t RTC_devInit(RTC_Device_t *dev)
{
    if (!dev)
        return -EFAULT;

    return rtc_init();
}

int32_t RTC_devStop(RTC_Device_t *dev)
{
    if (!dev)
        return -EFAULT;

    return rtc_stop();
}

int32_t RTC_devStartAlarm(RTC_Device_t *dev, RTC_ALARM_t alarm,
                         RTC_ALARM_MODE_t mode, uint32_t secs, void *cb)
{
    if (!dev)
        return -EFAULT;

    return alarm_start(alarm, mode, secs, cb);
}

int32_t RTC_devStopAlarm(RTC_Device_t *dev, RTC_ALARM_t alarm)
{
    if (!dev)
        return -EFAULT;

    return alarm_stop(alarm);
}
