/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file rx_device.c
 */

#include <string.h>
#include "rx_device.h"
#include "errno.h"

/**
 * The The RX device state
 */
RX_Dev_State_t RX_State = {
    .format = DCM_FMT_4BIT,
    .message_frame_size = 0,
    .buffer_size = RX_BUFF_SIZE,
    .buffer = {0},
    .cb = NULL,   
};

/**
 * The RF Setup for 433.92MHz reception
 */
static int32_t init_receiver(uint8_t bytes, DCM_FMT_t fmt)
{
    RX_SetState( RFOFF ); 
    RX_SetState( RFEN );                  /* RF Enabled */
    RX_ReinitDecoder();                   /* Clear the MSGRDY bit, starting reception */
    RX_SnifferMode( SNIFDIS );            /* Sniffer disable */
    RX_AgcControl( AGCEN, 0x1A );         /* AGC Enabled */
    RX_SetPllBiasTime( PLL48 );           /* PLL waits 48 clock cycles */
    RX_SetSleepTime( RFSLEEP10 );         /* Sleep time for sniffer is 10*1024 cycles */
    RX_SetWakeTime( WAKET12 );            /* Wake time from sniff is 12*1024 cycles (220ms for 3.58MHz crystal) */
    RFRX_PLL->HWORD[0] = 0x8073;          /* Set PLL divier */
    /* Hi side rejection enabled, fractional disabled, charge pump and loop filter */
    RX_Setup0( HISDEN, FEN, CPT01, 0x0A ); 
    /* 16 +DIV_LO2[1:0], Base band gain 00 = 0dB, LNA drain res. 1Kohm, LNA bias 1mA */
    RX_Setup1( DIV_LO2_16, BBGAIN0dB, LNADR1K, LNABIAS1mA);  
    /* Delay timer of the slicer, Attack time of the slicer, Time to allow slicer
        to fast bias, Symbol Decimation rate */
    RX_SlicerControl( ALPHA02, BETA02 , FT02 , SDR03 );      
    RX_SetMinTe( 2 );                     /* (1+Min_Te)*(12*16)/3.58e6 */
    RX_SetMaxTe( 22 );                    /* (1 + Min_Te + Max_Te)*(12*16)/3.58e6 */
    RX_SetMinGb( 5 );                     /* (1 + Min_Te + Max_Te + Min_Gb)*(12*16)/3.58e6 */
    /* PLL_EN (high if analog blocks enabled), RF always on Disabled, Number of edges 
       = 11 (to check in each sniff cycle before committing to a long wake cycle.  
       When in sniff mode, the first SNIFF_NE edges are tested for valid timing.  
       If any one of these first edges are badly timed, then the receiver will go to sleep. */
    RX_SetSnifRt( RTPLL, RFONDIS, 0x0B ); 
    RX_SetMinBitNumber(bytes * 8);      /* Minimum number of bits */
    RX_State.message_frame_size = bytes * 8;
    NVIC_ClearPendingIRQ(RFMSG_IRQn);   /* Clear RFMSG interrupt */
    NVIC_EnableIRQ (RFMSG_IRQn);        /* Enable RF reception interrupt */
    RX_State.format = fmt;
    return 0;
}

static int32_t stop_receiver(void)
{
    RX_SetState(RFOFF);
    return 0;
}

static int32_t get_recieved_data(uint8_t *data, uint8_t size)
{
    if (size <= RX_BUFF_SIZE) {
        memcpy(data, (uint8_t *)RX_State.buffer, size);
        return 0;
    } else
        return -EPERM;
}

static int32_t register_message_recive_interrupt(void *cb)
{
    RX_State.cb = (rx_cb_func_t)cb;
    return 0;
}

static int32_t unregister_message_recive_interrupt(void)
{
    RX_State.cb = NULL;
    return 0;
}

#ifdef RX_BASE
static RX_Device_t RX_Device = {
    .Init = init_receiver,
    .Stop = stop_receiver,
    .GetReceivedData = get_recieved_data,
    .RegisterMsgIRQ = register_message_recive_interrupt,
    .UnregisterMsgIRQ = unregister_message_recive_interrupt,
    .Name = "RX",
    .IRQ = RFMSG_IRQn,
    .state = &RX_State,
};
#endif

RX_Device_t *RX_devRequest(void)
{
#ifdef RX_BASE
    return (RX_Device_t *)&RX_Device;
#else
    return NULL,
#endif
}

void RFRE_Handler ( void )
{ 
}

void RFFE_Handler ( void )
{
}

static void read_rx_buffer(RX_Dev_State_t *state)
{
    int i;

    /* Read msg. into rx buffer in the reverse order.*/
    for ( i = 0; i < state->buffer_size; i++) {
        state->buffer[i] = RFRX_DATA->BYTE[state->buffer_size - 1 - i]; 
    }
}

void RFMSG_Handler( void )  			
{
    RX_Device_t *dev = &RX_Device;
    
    if ( RFRX->NUMB <= dev->state->message_frame_size + 4) {
        read_rx_buffer(dev->state);
        if (dev->state->cb)
            dev->state->cb(dev);
    }
    RX_SetState(RFEN);
    RX_ReinitDecoder();
}

int32_t RX_devInit(RX_Device_t *dev, uint8_t bytes, DCM_FMT_t fmt)
{
    if (!dev)
        return -EFAULT;
    return dev->Init(bytes, fmt);
}

int32_t RX_devGetReceivedData(RX_Device_t *dev, uint8_t *data, uint32_t len)
{
    if (!dev)
        return -EFAULT;
    return dev->GetReceivedData(data, len);
}

int32_t RX_devStop(RX_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    return dev->Stop();
}

int32_t RX_devRegisterCallback(RX_Device_t *dev, void *cb)
{
    if (!dev)
        return -EFAULT;
    return dev->RegisterMsgIRQ(cb);
}

int32_t RX_devUnregisterCallback(RX_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    return dev->UnregisterMsgIRQ();
}
