/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file spi_device.c
 */

#include <stdint.h>
#include <string.h>
#include "spi_device.h"
#include "errno.h"

/**
 * @brief The instance of SPI device
 */
#ifdef SPI_SFRS
const static SPI_Device_t SPI_Device = {
    .Enable = f_SPI_Enable,
    .Disable = f_SPI_Disable,
    .IrqEnable = f_SPI_IRQEnable,
    .IrqDisable = f_SPI_IRQDisable,
    .ReadData = f_SPI_ReadData,
    .GetStatus = f_SPI_GetStatus,
    .WriteData = f_SPI_WriteData,
    .SetMode = f_SPI_SetMode,
    .SetClockDivider = SPI_DevSetClockDivider,
    .SetPhase = f_SPI_SetPhase,
    .SetPolarity = f_SPI_SetPolarity,
    .IRQDelay = f_SPI_IRQDelay,
    .ClearIRQ = f_SPI_ClearIRQ,
    .IRQ = SPI_IRQn,
    .Name = "SPI",
};    
#endif

/**
 * @brief The table of SPI device instances supported by the platform.
 */
const static SPI_Device_t *SPI_DeviceTable[] = {
#ifdef SPI_SFRS
    &SPI_Device,
#endif
    NULL,
};

void SPI_DevSetClockDivider(uint8_t div)
{
    switch (div) {
    case SPI_DEV_CLOCK_DIV2:
        f_SPI_ClockDiv(E_SPI_CLOCK_DIV2, E_SPI_CLOCK_EXT_DIV2);
        break;
    case SPI_DEV_CLOCK_DIV4:
        f_SPI_ClockDiv(E_SPI_CLOCK_DIV4, E_SPI_CLOCK_EXT_DIV4);
        break;
    case SPI_DEV_CLOCK_DIV8:
        f_SPI_ClockDiv(E_SPI_CLOCK_DIV8, E_SPI_CLOCK_EXT_DIV8);
        break;
    case SPI_DEV_CLOCK_DIV16:
        f_SPI_ClockDiv(E_SPI_CLOCK_DIV16, E_SPI_CLOCK_EXT_DIV16);
        break;
    case SPI_DEV_CLOCK_DIV32:
        f_SPI_ClockDiv(E_SPI_CLOCK_DIV32, E_SPI_CLOCK_EXT_DIV32);
        break;
    case SPI_DEV_CLOCK_DIV64:
        f_SPI_ClockDiv(E_SPI_CLOCK_DIV64, E_SPI_CLOCK_EXT_DIV64);
        break;
    case SPI_DEV_CLOCK_DIV128:
        f_SPI_ClockDiv(E_SPI_CLOCK_DIV128, E_SPI_CLOCK_EXT_DIV128);
        break;
    case SPI_DEV_CLOCK_DIV256:
        f_SPI_ClockDiv(E_SPI_CLOCK_DIV256, E_SPI_CLOCK_EXT_DIV256);
        break;
    case SPI_DEV_CLOCK_DIV512:
        f_SPI_ClockDiv(E_SPI_CLOCK_DIV512, E_SPI_CLOCK_EXT_DIV512);
        break;
    case SPI_DEV_CLOCK_DIV1024:
        f_SPI_ClockDiv(E_SPI_CLOCK_DIV1024, E_SPI_CLOCK_EXT_DIV1024);
        break;
    case SPI_DEV_CLOCK_DIV2048:
        f_SPI_ClockDiv(E_SPI_CLOCK_DIV2048, E_SPI_CLOCK_EXT_DIV2048);
        break;
    case SPI_DEV_CLOCK_DIV4096:
        f_SPI_ClockDiv(E_SPI_CLOCK_DIV4096, E_SPI_CLOCK_EXT_DIV4096);
        break;
    default:
        break;
    }
}

SPI_Device_t *SPI_devRequest(const char *name)
{
    int8_t i;

    for (i = 0; i < sizeof(SPI_DeviceTable)/sizeof(SPI_DeviceTable[0]); i++) {
        if (strncmp(name, SPI_DeviceTable[i]->Name,
                   strlen(SPI_DeviceTable[i]->Name)) == 0)
            return SPI_DeviceTable[i];
    }
    return NULL;
}

int32_t SPI_devInit(SPI_Device_t *dev, SPI_Device_init_t *init )
{
    if(!dev || !init)
        return -EFAULT;
    dev->SetMode(init->Mode);
    dev->SetClockDivider(init->ClockDiv);
    dev->SetPhase(init->Phase);
    dev->SetPolarity(init->Polarity);
    dev->IRQDelay(init->IntDlyCnt);
    return 0;
}

int32_t SPI_devReadData(SPI_Device_t *dev, uint8_t cmd, uint8_t *data, uint32_t len)
{
    uint32_t i = 0;

    if(!dev || !data)
        return -EFAULT;

    dev->Enable();
    dev->WriteData(cmd);
    while(!f_SPI_TxFIFOEmpty());
    while(f_SPI_RxFIFOEmpty());
        dev->ReadData();
    while (i < len) {
        if (!f_SPI_TxFIFOFull()) {
            dev->WriteData(0);
            i++;
        }
        while(f_SPI_RxFIFOEmpty());
        *data = dev->ReadData();
        data++;
    }
    dev->Disable();
    return 0;
}

int32_t SPI_devWriteData(SPI_Device_t *dev, uint8_t cmd, uint8_t *data, uint32_t len)
{
    uint32_t i = 0, j = 0;
       
    if(!dev || !data)
        return -EFAULT;

    dev->Enable();
    i = 0;
    dev->WriteData(cmd);
    while (i < len) {
        while (f_SPI_TxFIFOFull());
        dev->WriteData(data[i]);
        i++;
    }
    while(!f_SPI_TxFIFOEmpty());
    for (j = 0; j < 10; j++);
    dev->Disable();
    return 0;
}

void SPI_Handler ( void )
{
    SPI_Device_t *dev = &SPI_Device;

    dev->ClearIRQ();
}
