/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file timer_device.c
 */

#include <stdint.h>
#include <stddef.h>
#include <string.h>
#include "timer_device.h"
#include "errno.h"

#ifdef TIMER0_SFRS
Timer_State_t timer0_state =
{
    .cb_func = NULL, .tick = 0,
};

/**
 * @brief The instance of timer0
 */
const static Timer_Device_t Timer0_Device =
{
    .Start = f_Timer0_Start,
    .Stop = f_Timer0_Stop,
    .LoadCounter = f_Timer0_LoadCounter,
    .IRQ = TIMER0_IRQn,
    .Num = 0,
    .State = &timer0_state,
    .Name = "TIMER0",
    .SFRS = (void *)TIMER0_SFRS,
};
#endif

#ifdef TIMER1_SFRS
Timer_State_t timer1_state = 
{
    .cb_func = NULL, .tick = 0,
};

/**
 * @brief The instance of timer1
 */
const static Timer_Device_t Timer1_Device = {
    .Start = f_Timer1_Start,
    .Stop = f_Timer1_Stop,
    .LoadCounter = f_Timer1_LoadCounter,
    .IRQ = TIMER1_IRQn,
    .Num = 1,
    .State = &timer1_state,
    .Name = "TIMER1",
    .SFRS = (void *)TIMER1_SFRS,
};
#endif

#ifdef TIMER2_SFRS
Timer_State_t timer2_state =
{
    .cb_func = NULL, .tick = 0,
};

/**
 * @brief The instance of timer2
 */
const static Timer_Device_t Timer2_Device = {
    .Start = f_Timer2_Start,
    .Stop = f_Timer2_Stop,
    .LoadCounter = f_Timer2_LoadCounter,
    .IRQ = TIMER2_IRQn,
    .Num = 2,
    .State = &timer2_state,
    .Name = "TIMER2",
    .SFRS = (void *)TIMER2_SFRS,
};    
#endif

/**
 * @brief The table of Timer device instances supported by the platform.
 */
const static Timer_Device_t *Timer_DeviceTable[] = {
#ifdef TIMER0
    &Timer0_Device,
#endif
#ifdef TIMER1
    &Timer1_Device,
#endif
#ifdef TIMER2
    &Timer2_Device,
#endif
};

int32_t Timer_devStart(Timer_Device_t *dev)
{
    if(!dev)
        return -EFAULT;
    dev->Start();
    return 0;
}

int32_t Timer_devStop(Timer_Device_t *dev)
{
    if(!dev)
        return -EFAULT;
    dev->Stop();
    return 0;
}

int32_t Timer_devSetMs(Timer_Device_t *dev, uint32_t ms)
{
    if(!dev)
        return -EFAULT;
    dev->LoadCounter((uint32_t)(-ms));
    return 0;
}

int32_t Timer_devRegisterIRQ(Timer_Device_t *dev, void *cb)
{
    if (!dev)
        return -EFAULT;
    dev->State->cb_func = (timer_cb_func_t)cb;
    return 0;
}

int32_t Timer_devUnregisterIRQ(Timer_Device_t *dev, void *cb)
{
    if (!dev)
        return -EFAULT;
    dev->State->cb_func = NULL;
    return 0;
}

int32_t Timer_devEnableIRQ(Timer_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    NVIC_EnableIRQ(dev->IRQ);
    return 0;
}

int32_t Timer_devDisableIRQ(Timer_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    NVIC_DisableIRQ(dev->IRQ);
    return 0;
}

int32_t Timer_devClearTick(Timer_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    dev->State->tick = 0;
    return 0;
}

int32_t Timer_devGetTick(Timer_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    return dev->State->tick;
}

void Timer0_Handler( void )
{
    if (timer0_state.cb_func != NULL)
        timer0_state.cb_func((Timer_Device_t *)&Timer0_Device);
    timer0_state.tick++;
}

void Timer1_Handler( void )
{
    if (timer1_state.cb_func != NULL)
        timer1_state.cb_func((Timer_Device_t *)&Timer1_Device);
    timer1_state.tick++;
}

void Timer2_Handler( void )
{
    if (timer2_state.cb_func != NULL)
        timer2_state.cb_func((Timer_Device_t *)&Timer2_Device);
    timer2_state.tick++;
}

Timer_Device_t *Timer_devRequest(const char *name)
{
    int8_t i;

    for (i = 0; i < sizeof(Timer_DeviceTable)/sizeof(Timer_DeviceTable[0]); i++) {
        if ((Timer_DeviceTable[i] != NULL) &&
            (strncmp(name, Timer_DeviceTable[i]->Name,
                    strlen(Timer_DeviceTable[i]->Name)) == 0))
            return Timer_DeviceTable[i];
    }
    return NULL;
}
