/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file tx_device.c
 */

#include <stdint.h>
#include <string.h>
#include "tx_device.h"
#include "rf_codec.h"
#include "errno.h"

#define BURST_BIT_SIZE_DFLT  160
#define WAIT_BIT_SIZE_DLFT   63 
#define TX_PREAMBLE     0x55
#define TX_START        0x08
#define TX_STOP         0x80
#define TX_MSG_OVERHEAD 8  /* the size for preamble, start, stop etc.*/

/**
 * @brief A structure to represent TX device state.
 */
typedef struct {
    DCM_FMT_t format;
    uint16_t burst_size;
    uint8_t wait_size;
    uint32_t buf_size;
    uint32_t tx_index;
    uint32_t tx_length;
    union {
        uint16_t hword[TX_BUFF_SIZE/2];
        uint8_t byte[TX_BUFF_SIZE];
    } tx_buffer;
    tx_cb_func_t cb;
    uint8_t busy;
} TX_Dev_State_t;
TX_Dev_State_t TX_State = {
    .format = DCM_FMT_4BIT,
    .burst_size = BURST_BIT_SIZE_DFLT,
    .wait_size = WAIT_BIT_SIZE_DLFT,
    .buf_size = TX_BUFF_SIZE,
    .tx_buffer = {0},
    .tx_index = 0,
    .tx_length = 0,
    .cb = NULL,
    .busy = 0,
};

int32_t tx_format_data(DCM_FMT_t fmt, uint8_t *data, uint32_t len)
{
    int32_t ret;
    uint8_t *buf = TX_State.tx_buffer.byte;
    
    if (len + TX_MSG_OVERHEAD > TX_BUFF_SIZE)
        return -EPERM;
    /* add preamble */
    *buf++ = TX_PREAMBLE;
    *buf++ = TX_PREAMBLE;
    /* add start bit */
    *buf++ = 0;
    *buf++ = TX_START;
    ret = DCM_Encoder(fmt, data, buf, len);
    if (ret > 0) {
        buf += ret;
        /* add stop bit */
        *buf++ = TX_STOP;
        *buf++ = 0;
        *buf++ = 0;
        *buf++ = 0;
        return ret + TX_MSG_OVERHEAD;
    } else
        return -EFAULT;
}

int32_t tx_init(TX_Config_t *cfg)
{
    if (cfg->burst_size)
        TX_State.burst_size = cfg->burst_size;
    if (cfg->wait_size)
        TX_State.wait_size = cfg->wait_size;
    if (cfg->format)
        TX_State.format = cfg->format;

    TX_TXconfig(TXCTRL0_MODUL_ASK, TXCTRL0_FRAC_EN, TXCTRL1_PEDES_10);
    TX_SetTiming(375, TX_State.burst_size, TX_State.wait_size);
    TX_SetFreq();
    TX_SetPowerLevel(0x00, 127);
    NVIC_ClearPendingIRQ( TX_RELOAD_IRQn );
    NVIC_ClearPendingIRQ( TX_DONE_IRQn );
    NVIC_EnableIRQ( TX_RELOAD_IRQn );
    NVIC_EnableIRQ( TX_DONE_IRQn );
    TX_State.burst_size = cfg->burst_size;
    TX_State.wait_size = cfg->wait_size;
    
    return 0;
}

static void tx_load_data(void)
{
    uint16_t data;

    data = TX_State.tx_buffer.hword[TX_State.tx_index];
    data = ( ( data & 0xff ) << 8) | ( data >> 8 );
    TX_LoadData(data);
    TX_State.tx_index++;
}

static int32_t tx_send_data(uint8_t *data, uint32_t len)
{
    int32_t ret;

    if (len > TX_State.buf_size)
    return -EPERM;

    ret = tx_format_data(TX_State.format, data, len);
    if (ret < 0)
        return -EFAULT;
    TX_State.tx_index = 0;
    TX_State.burst_size = ret * 8;
    TX_State.tx_length = ret / 2;
    TX_SetMsgFormat(TX_State.burst_size, TX_State.wait_size);
    
    TX_State.busy = 1;
    tx_load_data();
    TX_StartTransmission();
    TX_En_NextBurst();

    return 0;
}

int32_t tx_stop(void)
{
    TX_StopTransmission();
    TX_Dis_NextBurst();
    return 0;
}  

static int32_t tx_is_busy(void)
{
    return (TX_State.busy);
}

#ifdef TX_BASE
TX_Device_t TX_Device = {
    .SendData = tx_send_data,
    .Stop = tx_stop,
    .Init = tx_init,
    .RegisterCallback = 0,
    .UnregisterCallback = 0,
    .IsBusy = tx_is_busy,
    .IRQ_Reload = TX_RELOAD_IRQn,
    .IRQ_Done = TX_DONE_IRQn,
    .Name = "TX",
};
#endif

TX_Device_t *TX_devRequest(void)
{
#ifdef TX_BASE
    return (TX_Device_t *)&TX_Device;
#else
    return NULL,
#endif
}

int32_t TX_devInit(TX_Device_t *dev, TX_Config_t *cfg)
{
    if (!dev)
        return -EFAULT;
    return dev->Init(cfg);
}

int32_t TX_devSendData(TX_Device_t *dev, uint8_t *data, uint32_t size)
{
    if (!dev)
        return -EFAULT;
    return dev->SendData(data, size);
}

int32_t TX_devStop(TX_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    return dev->Stop();
}  

int32_t TX_devDataPendingTX(TX_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    return dev->IsBusy();
}

int32_t TX_devRegisterCallback(TX_Device_t *dev, void *cb)
{
    if (!dev)
        return -EFAULT;
    return dev->RegisterCallback(cb);
}

int32_t TX_devUnregisterCallback(TX_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    return dev->UnregisterCallback();
}

void TX_DONE_Handler( void )
{
    if (TX_State.busy == 0) {
        TX_StopTransmission();
        TX_Dis_NextBurst();
    }
}

void TX_RELOAD_Handler( void ) 
{
    if (TX_State.busy)
        tx_load_data();
    if (TX_State.tx_index >= TX_State.tx_length)
        TX_State.busy = 0;
}

