/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file uart_device.c
 */

#include <stdint.h>
#include <string.h>
#include "uart_device.h"
#include "errno.h"

/**
 * @brief The instances of UART device
 */
#ifdef UART_SFRS
extern UART_Dev_State_t UART_State;
static UART_Device_t UART_Device = {
    .IntStatus = f_UART_IntStatus,
    .Enable = f_UART_Enable,
    .Disable = f_UART_Disable,
    .RxFiFoReset = f_UART_RxFiFoReset,
    .TxFiFoReset = f_UART_TxFiFoReset,
    .LineStatus = f_UART_LineStatus,

    .BaudRateDiv = f_UART_BaudRateDiv,
    .EnableRxIRQ = f_UART_EnableRxIRQ,
    .EnableTxIRQ = f_UART_EnableTxIRQ,
    .EnableRxTimeoutIRQ = f_UART_EnableRxTimeoutIRQ,
    .EnableRxErrorIRQ = f_UART_EnableRxErrorIRQ,
    .DisableRxIRQ = f_UART_DisableRxIRQ,
    .DisableTxIRQ = f_UART_DisableTxIRQ,
    .DisableRxTimeoutIRQ = f_UART_DisableRxTimeoutIRQ,
    .DisableRxErrorIRQ = f_UART_DisableRxErrorIRQ,
   
    .TxEmpty = f_UART_TxEmpty,
    .RxDataReady = f_UART_RxDataReady,
    .TxData = f_UART_TxData,
    .RxData = f_UART_RxData,
    .SetDataSize = f_UART_SetDataSize,
    .SetStopBit = f_UART_SetStopBit,
    .SetEvenParityBit = f_UART_SetEvenParityBit,
    .SetOddParityBit = f_UART_SetOddParityBit,
    .EnableParity = f_UART_EnableParity,
    .DisableParity = f_UART_DisableParity,
    .EnableStickyParity = f_UART_EnableStickyParity,
    .DisableStickyParity = f_UART_DisableStickyParity,
    .EnableBreak = f_UART_EnableBreak,
    .DisableBreak = f_UART_DisableBreak,
    .EnableLoopBack = f_UART_EnableLoopBack,
    .DisableLoopBack = f_UART_DisableLoopBack,
    .SetInterruptDelay = f_UART_SetInterruptDelay,

    .IRQ = UART_IRQn,
    .Name = "UART",
    .state = &UART_State,
    .SFRS = (void *)UART_SFRS,
};
#endif

/**
 * @brief The table of UART device instances supported by the platform.
 */
const static UART_Device_t *UART_DeviceTable[] = {
#ifdef UART_SFRS
    &UART_Device,
#endif
    NULL,
};

/**
 * The clock divider settings for each UART baud rate at 12M system clock.
 */
static const uint16_t board_rate_map[UART_DEV_BAUDRATE_MAX+1] = {
    [UART_DEV_BAUDRATE_300]	= 6249,
    [UART_DEV_BAUDRATE_600]	= 3124,
    [UART_DEV_BAUDRATE_1200]	= 1562,
    [UART_DEV_BAUDRATE_2400]	= 780,
    [UART_DEV_BAUDRATE_4800]	= 390,
    [UART_DEV_BAUDRATE_9600]	= 194,
    [UART_DEV_BAUDRATE_19200]	= 97,
    [UART_DEV_BAUDRATE_38400]	= 48,
    [UART_DEV_BAUDRATE_57600]	= 32,
    [UART_DEV_BAUDRATE_115200]	= 15,
    [UART_DEV_BAUDRATE_230400]  = 7,
};

/**
 * The The UART device state
 */
UART_Dev_State_t UART_State = {
    .buf_size = UART_BUFF_SIZE,
    .tx_buffer = {0},
    .rx_buffer = {0},
    .rx_outptr = 0,
    .rx_inptr = 0,
    .tx_outptr = 0,
    .tx_inptr = 0,
    .tx_init = 0,
    .cb_size = 0,
    .cb = NULL,   
};

void uart_reset_tx(UART_Device_t *dev);
void uart_reset_rx(UART_Device_t *dev);

UART_Device_t *UART_devRequest(const char *name)
{
    int8_t i;

    for (i = 0; i < sizeof(UART_DeviceTable)/sizeof(UART_DeviceTable[0]); i++) {
        if ((UART_DeviceTable[i] != NULL) &&
            (strncmp(name, UART_DeviceTable[i]->Name,
                   strlen(UART_DeviceTable[i]->Name)) == 0))
            return UART_DeviceTable[i];
    }
    return NULL;
}

int32_t UART_devInit(UART_Device_t *dev, UART_devBaudRate_t rate, UART_Dev_LineInitData_t *init)
{
    if (!dev)
        return -EFAULT;
    dev->SetDataSize(init->DataSize);
    dev->SetStopBit(init->StopBits);
    
    if (init->ParityBit == UART_DEV_PARITY_BIT_EVEN)
        dev->SetEvenParityBit();
    else
        dev->SetOddParityBit();
    
    if (init->ParityEn)
        dev->EnableParity();
    else
        dev->DisableParity();
    
    if (init->StickyParityEn)
        dev->EnableStickyParity();
    else
        dev->DisableStickyParity();
    
    if (init->BreakEn)
        dev->EnableBreak();
    else
        dev->DisableBreak();
    
    if (init->LoopBackEn)
        dev->EnableLoopBack();
    else
        dev->DisableLoopBack();
    
    dev->EnableRxIRQ();
    dev->EnableTxIRQ();
    dev->EnableRxTimeoutIRQ();
    dev->EnableRxErrorIRQ();
 
    dev->BaudRateDiv(board_rate_map[rate]);

    dev->Enable();
    NVIC_EnableIRQ(dev->IRQ);
    uart_reset_tx(dev);
    uart_reset_rx(dev);
    return 0;
}

void uart_reset_tx(UART_Device_t *dev)
{
    dev->state->tx_outptr = 0;
    dev->state->tx_inptr = 0;
    dev->EnableTxIRQ();
}

void uart_reset_rx(UART_Device_t *dev)
{
    dev->state->rx_outptr = 0;
    dev->state->rx_inptr = 0;
    dev->EnableRxIRQ();
}

uint32_t uart_read(UART_Device_t *dev, uint8_t *data)
{
    if (dev->state->rx_inptr == dev->state->rx_outptr)
        return 0;
    *data = dev->state->rx_buffer[dev->state->rx_outptr];
    dev->state->rx_outptr = (dev->state->rx_outptr + 1) % dev->state->buf_size;
    return 1;
}

void uart_write(UART_Device_t *dev, uint8_t data)
{
    while (((dev->state->tx_inptr+1) % dev->state->buf_size) == dev->state->tx_outptr) {};
    if ((dev->state->tx_inptr == dev->state->tx_outptr) && (dev->state->tx_init == 0)) {
        dev->TxData(data);
        dev->state->tx_init = 1;
    }
    dev->state->tx_buffer[dev->state->tx_inptr] = data;
    dev->state->tx_inptr = (dev->state->tx_inptr + 1) % dev->state->buf_size;
}

int32_t UART_devDataReceived(UART_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    return (dev->state->buf_size + dev->state->rx_inptr -
           dev->state->rx_outptr) % dev->state->buf_size;
}

int32_t UART_devDataPendingTX(UART_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    return (dev->state->buf_size + dev->state->tx_inptr -
            dev->state->tx_outptr) % dev->state->buf_size;
}

int32_t UART_devSendByte(UART_Device_t *dev, uint8_t *data, uint32_t len)
{
    uint32_t i;

    if (!dev)
        return -EFAULT;
    for (i = 0; i < len; i++)
        uart_write(dev, data[i]);
    return 0;
}

int32_t UART_devReceiveByte(UART_Device_t *dev, uint8_t *data, uint32_t len)
{
    uint32_t i = 0, ret;

    if (!dev)
        return -EFAULT;
    while (i < len) {
        ret = uart_read(dev, data);
        if (ret) {
            data++;
            i++;
        }
    }
    return 0;
}

int32_t UART_devClearBuffer(UART_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    dev->state->tx_outptr = 0;
    dev->state->tx_inptr = 0;
    dev->state->rx_outptr = 0;
    dev->state->rx_inptr = 0;
    return 0;
}

int32_t UART_devRegisterCallback(UART_Device_t *dev, uint32_t size, void *cb)
{
    if (!dev)
        return -EFAULT;
    dev->state->cb_size = size;
    dev->state->cb = (uart_cb_func_t)cb;
    return 0;
}

void uart_tx_isr_handler(UART_Device_t *dev)
{
    if (dev->state->tx_init != 0) {
        dev->state->tx_outptr = (dev->state->tx_outptr + 1) % dev->state->buf_size;
        dev->state->tx_init = 0;
    }

    if (dev->state->tx_inptr != dev->state->tx_outptr)
    {
        while (!dev->TxEmpty());
        dev->TxData(dev->state->tx_buffer[dev->state->tx_outptr]);
        dev->state->tx_outptr = (dev->state->tx_outptr + 1) % dev->state->buf_size;
    }
}

void uart_rx_isr_handler(UART_Device_t *dev)
{
    dev->state->rx_buffer[dev->state->rx_inptr] = dev->RxData();
    dev->state->rx_inptr = (dev->state->rx_inptr + 1) % dev->state->buf_size;
    if((dev->state->cb) &&
       (((dev->state->buf_size + dev->state->rx_inptr - dev->state->rx_outptr)
        % dev->state->buf_size) >= dev->state->cb_size))
        dev->state->cb(dev);
}

int32_t UART_devCheckError(UART_Device_t *dev)
{
    uint32_t status;

    if (!dev)
        return -EFAULT;
    status = dev->LineStatus();
    return status & (E_UART_STATUS_ERROR & E_UART_STATUS_OVERRUN_ERROR &
                     E_UART_STATUS_FRAME_ERROR & E_UART_STATUS_PARITY_ERROR &
                         E_UART_STATUS_BREAKINT_ERROR);
}

int32_t UART_devResetDevice(UART_Device_t *dev)
{
    if (!dev)
        return -EFAULT;
    dev->Disable();
    dev->Enable();
    dev->state->tx_init = 0;
    return 0;
}

void UART_Handler (void)
{
    UART_Device_t *dev = &UART_Device;
    uint8_t status;
    uint8_t status_new;
    uint8_t line_status;

    NVIC_DisableIRQ(dev->IRQ);
    status = dev->IntStatus();
    line_status = dev->LineStatus();

    while (1) {
        switch (status) {
        case E_UART_INT_STATUS_RX_READY:
            uart_rx_isr_handler(dev);
            break;
        case E_UART_INT_STATUS_TX_COMPLETE:
            uart_tx_isr_handler(dev);
            break;
        case E_UART_INT_STATUS_RX_TIMEOUT:
            break;
        case E_UART_INT_STATUS_RX_ERROR:
            break;
        case E_UART_INT_STATUS_NONE:
        default:
            break;
        }
        status_new = dev->IntStatus();
        if ((status_new != E_UART_INT_STATUS_RX_READY) && 
            (status_new != E_UART_INT_STATUS_TX_COMPLETE))
            break;
        status = status_new;
    }
    NVIC_EnableIRQ(dev->IRQ);
}
