/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file adc_sfrs.h
 */
 
#ifndef __ADC_SFRS_H__
#define __ADC_SFRS_H__

/**
 * @brief A structure to represent Special Function Registers for ADC block.
 */
typedef struct {
    union {
        struct {
            uint8_t ADCDIV:6;
            uint8_t :1;
            uint8_t SWMODE:1;
        };
        uint8_t BYTE;    
    } ADCCLKDIV;
    union {
        struct {
            uint8_t ADCCH:5;
            uint8_t :3;
        };
        uint8_t BYTE;    
    } CHANNELS;
    union {
        struct {
            uint8_t ADCSTART:1;
            uint8_t :7;
        };
        uint8_t START;    
    } START;
    uint8_t RESULT;    
    uint8_t Reserved0[98297];
    union {
        struct {
            uint8_t ADCREFL:4;
            uint8_t ADCREFH:4;
        };
        uint8_t BYTE;    
    } TRIM0;
    union {
        struct {
            uint8_t ADCPGN:4;
            uint8_t ADCREFS:1;
            uint8_t ADCCAL:1;
            uint8_t ADCSW:2;
        };
        uint8_t BYTE;    
    } TRIM1;
} ADC_SFRS_t;

/**
 * @brief The starting address of ADC block.
 */
#define ADC_SFRS       ((__IO ADC_SFRS_t *) (0x5000000B))

#define E_ADC_CLK_DIV2          0
#define E_ADC_CLK_DIV4          1
#define E_ADC_CLK_DIV64         0x1F

#define E_ADC_CHANNEL_GPIOD0    0
#define E_ADC_CHANNEL_GPIOD1    1
#define E_ADC_CHANNEL_GPIOD2    2
#define E_ADC_CHANNEL_GPIOD3    3
#define E_ADC_CHANNEL_GPIOD4    4
#define E_ADC_CHANNEL_GPIOD5    5
#define E_ADC_CHANNEL_GPIOD6    6
#define E_ADC_CHANNEL_GPIOD7    7
#define E_ADC_CHANNEL_GPIOE0    8
#define E_ADC_CHANNEL_GPIOE1    9
#define E_ADC_CHANNEL_GPIOE2    10
#define E_ADC_CHANNEL_GPIOE3    11
#define E_ADC_CHANNEL_GPIOE4    12
#define E_ADC_CHANNEL_GPIOE5    13
#define E_ADC_CHANNEL_GPIOE6    14
#define E_ADC_CHANNEL_GPIOE7    15
#define E_ADC_CHANNEL_GPIOF0    16
#define E_ADC_CHANNEL_GPIOF1    17
#define E_ADC_CHANNEL_GPIOF2    18
#define E_ADC_CHANNEL_VSUPPLY   19

/**
 * @brief Select ADC channel.
 *
 * @param channel The enumeration of ADC channel.
 */
static __INLINE void f_ADC_SelectChannel(uint8_t channel)
{
   ADC_SFRS->CHANNELS.ADCCH = channel;
}

/**
 * @brief Set ADC clock divider.
 *
 * @param div The enumeration for ADC clock divider.
 */
static __INLINE void f_ADC_ClkDiv(uint8_t div)
{
    ADC_SFRS->ADCCLKDIV.ADCDIV = div;
}

/**
 * @brief Set ADC sample cycle.
 *
 * @param cycle The enumeration for sample cycle.
 */
static __INLINE void f_ADC_SampleCycle(uint8_t cycle)
{
}

/**
 * @brief Read ADC result.
 *
 * @return The result of last ADC conversion.
 */
static __INLINE uint8_t f_ADC_Read(void)
{
    return ADC_SFRS->RESULT;
}

/**
 * @brief Start ADC conversion.
 */
static __INLINE void f_ADC_Start(void)
{
    ADC_SFRS->START.ADCSTART = 1;
}

/**
 * @brief Check ADC progress.
 *
 * @return The status of a ADC conversion.
 */
static __INLINE uint8_t f_ADC_ConversionInProgress()
{
    return ADC_SFRS->START.ADCSTART;
}

/**
 * @brief Set ADC high reference level.
 *
 * @param val The setting for high reference.
 */
static __INLINE void f_ADC_ReferenceHigh(uint8_t val)
{
    ADC_SFRS->TRIM0.ADCREFH = val;
}

/**
 * @brief Set ADC low reference level.
 *
 * @param val The setting for low reference.
 */
static __INLINE void f_ADC_ReferenceLow(uint8_t val)
{
    ADC_SFRS->TRIM0.ADCREFL = val;
}

/**
 * @brief Set ADC reference gain.
 *
 * @param val The setting for reference gain.
 */
static __INLINE void f_ADC_ReferenceGain(uint8_t val)
{
    ADC_SFRS->TRIM1.ADCPGN = val;
}

/**
 * @brief Select VDD as ADC reference source.
 */
static __INLINE void f_ADC_SetRefSourceVDD(void)
{
    ADC_SFRS->TRIM1.ADCREFS = 1;
}

/**
 * @brief Select band gap as ADC reference source.
 */
static __INLINE void f_ADC_SetRefSourceBandGap(void)
{
    ADC_SFRS->TRIM1.ADCREFS = 0;
}

/**
 * @brief Enable ADC streaming mode.
 */
static __INLINE void f_ADC_EnableStreamingMode(void)
{
}

/**
 * @brief Disable ADC streaming mode.
 */
static __INLINE void f_ADC_DisableStreamingMode(void)
{
}

/**
 * @brief Enable ADC calibration mode.
 */
static __INLINE void f_ADC_EnableCalibrationMode(void)
{
    ADC_SFRS->TRIM1.ADCCAL = 1;
}

/**
 * @brief Disable ADC calibration mode.
 */
static __INLINE void f_ADC_DisableCalibrationMode(void)
{
    ADC_SFRS->TRIM1.ADCCAL = 0;
}

/**
 * @brief Enable ADC offset calibration mode.
 */
static __INLINE void f_ADC_EnableOffsetCalMode(void)
{
    ADC_SFRS->TRIM1.ADCSW = 1;
}

/**
 * @brief Enable ADC double sample mode.
 */
static __INLINE void f_ADC_EnableDoubleSample(void)
{
    ADC_SFRS->TRIM1.ADCSW = 2;
}

/**
 * @brief Disable ADC double sample mode.
 */
static __INLINE void f_ADC_DisableDoubleSample(void)
{
    ADC_SFRS->TRIM1.ADCSW = 0;
}

#endif /* __ADC_SFRS_H__ */