/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file clock_sfrs.h
 */

#ifndef __CLOCK_SFRS_H__
#define __CLOCK_SFRS_H__

/**
 * @brief A structure to represent Special Function Registers for Clock.
 */
typedef struct {
    union {
        struct {
            uint8_t     CKSEL:2;
            uint8_t     RCEN:1;
            uint8_t     XOEN:1;
            uint8_t     XOEN32K:1;
            uint8_t     XOMON:1;
            uint8_t     CKD:2;              
        };
        uint8_t BYTE;
    } PUMCLK;
    uint8_t RESERVED0[98322];
    union {
        struct {
            uint8_t     TRIM:5;
            uint8_t     RESERVED:3;
        };
        uint8_t BYTE;
    } RCC10KTRIM;
    union {
        struct {
            uint8_t     RESERVED:5;
            uint8_t     TRIM:3;
        };
        uint8_t BYTE;
    } RCTRIM0;
    union {
        struct {
            uint8_t     TRIM:4;
            uint8_t     RESERVED:4;
        };
        uint8_t BYTE;
    } RCTRIM1;
} CLOCK_SFRS_t;

/**
 * @brief The starting address of Clock SFRS.
 */
#define CLOCK_SFRS      ((__IO CLOCK_SFRS_t *)(ASIC_7B_BASE + 0))

#define	E_CLOCK_SOURCE_RC10KHZ		0x00
#define	E_CLOCK_SOURCE_RC12MHZ		0x01
#define	E_CLOCK_SOURCE_XTCLOCK		0x02
#define	E_CLOCK_SOURCE_RCAUX		0x03

/**
 * @brief Select clock source.
 *
 * param src The enumeration of clock source.
 */
static __INLINE void f_CLOCK_SelectSource(uint8_t src)
{
    CLOCK_SFRS->PUMCLK.CKSEL = src;
}

/**
 * @brief Get clock source.
 *
 * return The enumeration of clock source.
 */
static __INLINE uint8_t f_CLOCK_GetClockSource(void)
{
    return CLOCK_SFRS->PUMCLK.CKSEL;
}

/**
 * @brief Enable crystal clock source.
 */
static __INLINE void f_CLOCK_EnableXTClock(void)
{
    CLOCK_SFRS->PUMCLK.XOEN = 1;
}

/**
 * @brief Disable crystal clock source.
 */
static __INLINE void f_CLOCK_DisableXTClock(void)
{
    CLOCK_SFRS->PUMCLK.XOEN = 0;
}

/**
 * @brief Enable 32K crystal clock source.
 */
static __INLINE void f_CLOCK_EnableXTClock32K(void)
{
    CLOCK_SFRS->PUMCLK.XOEN32K = 1;
}

/**
 * @brief Disable 32K crystal clock source.
 */
static __INLINE void f_CLOCK_DisableXTClock32K(void)
{
    CLOCK_SFRS->PUMCLK.XOEN32K = 0;
}

/**
 * @brief Check if crystal clock source enabled.
 *
 * return The status of crystal clock source.
 */
static __INLINE uint8_t f_CLOCK_IsXTClockEnabled(void)
{
    return CLOCK_SFRS->PUMCLK.XOEN;
}

/**
 * @brief Enable RC clock source.
 */
static __INLINE void f_CLOCK_EnableRCClock(void)
{
    CLOCK_SFRS->PUMCLK.RCEN = 1;
}

/**
 * @brief Disable RC clock source.
 */
static __INLINE void f_CLOCK_DisableRCClock(void)
{
    CLOCK_SFRS->PUMCLK.RCEN = 0;
}

/**
 * @brief Check if RC clock source enabled.
 *
 * return The status of RC clock source.
 */
static __INLINE uint8_t f_CLOCK_IsRCClockEnabled(void)
{
    return CLOCK_SFRS->PUMCLK.RCEN;
}

/**
 * @brief Trim RC clock.
 *
 * @param val The trim setting for RC clock.
 */
static __INLINE void f_CLOCK_RCTrim(uint8_t val)
{
    CLOCK_SFRS->RCTRIM0.TRIM = val;
    CLOCK_SFRS->RCTRIM1.TRIM = val >> 3;
}

/**
 * @brief Trim 10K RC clock.
 *
 * @param val The trim setting for 10K RC clock.
 */
static __INLINE void f_CLOCK_RC10KTrim(uint8_t val)
{
    CLOCK_SFRS->RCC10KTRIM.TRIM = val;
}

#endif /* __CLOCK_SFRS_H__ */
