/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file gpio_sfrs.h
 */

#ifndef __GPIO_SFRS_H__
#define __GPIO_SFRS_H__

#include <stdbool.h>
#include <stdint.h>

#define E_GPIO_PIN_MODE_OFF     0
#define E_GPIO_PIN_MODE_ON      1
#define E_GPIO_PIN_NOINVERT     0
#define E_GPIO_PIN_INVERT       1
#define E_GPIO_PIN_NOSWAP       0
#define E_GPIO_PIN_SWAP         1

#define E_GPIOD_BITWIDTH        8
#define E_GPIOE_BITWIDTH        8
#define E_GPIOF_BITWIDTH        3

/**
 * @brief A structure to represent GPIO configuration.
 */
typedef struct {
    uint8_t PULLDOWN:1;
    uint8_t PULLUP:1;
    uint8_t READEN:1;
    uint8_t OUTPUTEN:1;
    uint8_t :4;
} GPIO_CONFIG_t;

/**
 * @brief A structure to represent Special Function Registers for GPIOD block.
 */
typedef struct {
    uint8_t     PORT;
    uint8_t     RESERVED0[5];
    uint8_t     INTEN;
    uint8_t     RESERVED1[98231];
    GPIO_CONFIG_t CFG[E_GPIOD_BITWIDTH];
} GPIOD_SFRS_t;
#define GPIOD_SFRS ((__IO GPIOD_SFRS_t *)(0x50000062))

/**
 * @brief A structure to represent Special Function Registers for GPIOE block.
 */
typedef struct {
    uint8_t     PORT;
    uint8_t     RESERVED0[5];
    uint8_t     INTEN;
    uint8_t     RESERVED1[98238];
    GPIO_CONFIG_t CFG[E_GPIOE_BITWIDTH];
} GPIOE_SFRS_t;
#define GPIOE_SFRS ((__IO GPIOE_SFRS_t *)(0x50000063))

/**
 * @brief A structure to represent Special Function Registers for GPIOF block.
 */
typedef struct {
    uint8_t     PORT;
    uint8_t     RESERVED0[5];
    uint8_t     INTEN;
    uint8_t     RESERVED1[98245];
    GPIO_CONFIG_t CFG[E_GPIOF_BITWIDTH];
} GPIOF_SFRS_t;
#define GPIOF_SFRS ((__IO GPIOF_SFRS_t *)(0x50000064))

/**
 * @brief A structure to represent Special Function Registers for GPIO Mode.
 */
typedef struct {
    uint8_t UTXPOL:1;
    uint8_t MDUART:1;
    uint8_t MDSPI:1;
    uint8_t :1;
    uint8_t UPSWAP:1;
    uint8_t :3;
} GPIO_MODE_SFRS_t;
#define GPIO_MODE_SFRS ((__IO GPIO_MODE_SFRS_t *)(0x50000065))

/**
 * @brief Enable/disable pull up on GPIOD
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOD_EnablePullUp(uint8_t pos, bool bit);

/**
 * @brief Enable/disable pull down on GPIOD
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOD_EnablePullDown(uint8_t pos, bool bit);

/**
 * @brief Enable/disable output on GPIOD
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOD_EnableOutput(uint8_t pos, bool bit);

/**
 * @brief Enable/disable Input on GPIOD
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOD_EnableInput(uint8_t pos, bool bit);

/**
 * @brief Enable/disable interrupt on GPIOD
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOD_EnableInterrupt(uint8_t pos, bool bit);

/**
 * @brief Read pin state on GPIOD
 *
 * @param pos The pin position.
 * @return The pin state.
 */
uint8_t GPIOD_Read(uint8_t pos);

/**
 * @brief Set pin state on GPIOD
 *
 * @param pos The pin position.
 * @param bit The pin state.
 */
void GPIOD_Set(uint8_t pos, uint8_t bit);

/**
 * @brief Enable/disable pull up bits on GPIOD
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOD_EnablePortPullUp(bool bit, uint8_t mask);

/**
 * @brief Enable/disable pull down on GPIOD
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOD_EnablePortPullDown(bool bit, uint8_t mask);

/**
 * @brief Enable/disable output enable bits on GPIOD
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOD_EnablePortOutput(bool bit, uint8_t mask);

/**
 * @brief Enable/disable input enable bits on GPIOD
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOD_EnablePortInput(bool bit, uint8_t mask);

/**
 * @brief Enable/disable interrupt enable bits on GPIOD
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOD_EnablePortInterrupt(bool bit, uint8_t mask);

/**
 * @brief Set port value on GPIOD
 *
 * @param val The value to be set to GPIO port.
 * @param mask The bit mask for the pins to be configured.
 * @return None.
 */
void GPIOD_PortSet(uint8_t val, uint8_t mask);

/**
 * @brief Read port value from GPIOD
 *
 * @param mask The bit mask for the pins to be read.
 * @return The masked port value from GPIOD.
 */
uint8_t GPIOD_PortRead(uint8_t mask);

/**
 * @brief Enable/disable pull up on GPIOE
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOE_EnablePullUp(uint8_t pos, bool bit);

/**
 * @brief Enable/disable pull down on GPIOE
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOE_EnablePullDown(uint8_t pos, bool bit);

/**
 * @brief Enable/disable output on GPIOE
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOE_EnableOutput(uint8_t pos, bool bit);

/**
 * @brief Enable/disable Input on GPIOE
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOE_EnableInput(uint8_t pos, bool bit);

/**
 * @brief Enable/disable interrupt on GPIOE
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOE_EnableInterrupt(uint8_t pos, bool bit);

/**
 * @brief Read pin state on GPIOE
 *
 * @param pos The pin position.
 * @return The pin state.
 */
uint8_t GPIOE_Read(uint8_t pos);

/**
 * @brief Set pin state on GPIOE
 *
 * @param pos The pin position.
 * @param bit The pin state.
 */
void GPIOE_Set(uint8_t pos, uint8_t bit);

/**
 * @brief Enable/disable pull up bits on GPIOE
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOE_EnablePortPullUp(bool bit, uint8_t mask);

/**
 * @brief Enable/disable pull down on GPIOE
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOE_EnablePortPullDown(bool bit, uint8_t mask);

/**
 * @brief Enable/disable output enable bits on GPIOE
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOE_EnablePortOutput(bool bit, uint8_t mask);

/**
 * @brief Enable/disable input enable bits on GPIOE
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOE_EnablePortInput(bool bit, uint8_t mask);

/**
 * @brief Enable/disable interrupt enable bits on GPIOE
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOE_EnablePortInterrupt(bool bit, uint8_t mask);

/**
 * @brief Set port value on GPIOE
 *
 * @param val The value to be set to GPIO port.
 * @param mask The bit mask for the pins to be configured.
 * @return None.
 */
void GPIOE_PortSet(uint8_t val, uint8_t mask);

/**
 * @brief Read port value from GPIOE
 *
 * @param mask The bit mask for the pins to be read.
 * @return The masked port value from GPIOE.
 */
uint8_t GPIOE_PortRead(uint8_t mask);

/**
 * @brief Enable/disable pull up on GPIOF
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOF_EnablePullUp(uint8_t pos, bool bit);

/**
 * @brief Enable/disable pull down on GPIOF
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOF_EnablePullDown(uint8_t pos, bool bit);

/**
 * @brief Enable/disable output on GPIOF
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOF_EnableOutput(uint8_t pos, bool bit);

/**
 * @brief Enable/disable Input on GPIOF
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOF_EnableInput(uint8_t pos, bool bit);

/**
 * @brief Enable/disable interrupt on GPIOF
 *
 * @param pos The pin position.
 * @param bit The enable bit, true for enable, false for disable.
 */
void GPIOF_EnableInterrupt(uint8_t pos, bool bit);

/**
 * @brief Read pin state on GPIOF
 *
 * @param pos The pin position.
 * @return The pin state.
 */
uint8_t GPIOF_Read(uint8_t pos);

/**
 * @brief Set pin state on GPIOF
 *
 * @param pos The pin position.
 * @param bit The pin state.
 */
void GPIOF_Set(uint8_t pos, uint8_t bit);

/**
 * @brief Enable/disable pull up bits on GPIOF
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOF_EnablePortPullUp(bool bit, uint8_t mask);

/**
 * @brief Enable/disable pull down on GPIOF
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOF_EnablePortPullDown(bool bit, uint8_t mask);

/**
 * @brief Enable/disable output enable bits on GPIOF
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOF_EnablePortOutput(bool bit, uint8_t mask);

/**
 * @brief Enable/disable input enable bits on GPIOF
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOF_EnablePortInput(bool bit, uint8_t mask);

/**
 * @brief Enable/disable interrupt enable bits on GPIOF
 *
 * @param bit The enable bit, true for enable, false for disable.
 * @param mask The bit mask for the bits to be configured.
 * @return None.
 */
void GPIOF_EnablePortInterrupt(bool bit, uint8_t mask);

/**
 * @brief Set port value on GPIOF
 *
 * @param val The value to be set to GPIO port.
 * @param mask The bit mask for the pins to be configured.
 * @return None.
 */
void GPIOF_PortSet(uint8_t val, uint8_t mask);

/**
 * @brief Read port value from GPIOF
 *
 * @param mask The bit mask for the pins to be read.
 * @return The masked port value from GPIOF.
 */
uint8_t GPIOF_PortRead(uint8_t mask);

/**
 * @brief Config GPIO pin to UART mode
 *
 * @param mode GPIO mode or non GPIO mode (UART mode).
 * @param swap The pin swap setting.
 * @param invert The polarity setting.
 */
void GPIO_ConfigUART(uint8_t mode, uint8_t swap, uint8_t invert);

/**
 * @brief Config GPIO pin to SPI mode
 *
 * @param mode GPIO mode or non GPIO mode (SPI mode).
 */
void GPIO_ConfigSPI(uint8_t mode);

#endif /* __GPIO_SFRS_H__ */