/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file pmu_sfrs.h
 */

#ifndef __PMU_SFRS_H__
#define __PMU_SFRS_H__

/**
 * @brief A structure to represent Special Function Registers for PMU.
 */
typedef struct {
    union {
        struct {
            uint8_t PORF:1;
            uint8_t BROUT:1;
            uint8_t RESERVED:3;
            uint8_t DLEEP:1;
            uint8_t SWRST:1;
            uint8_t HWRST:1;
        };
        uint8_t BYTE;
    } PMURST;
    union {
        struct {
            uint8_t BOUTVALUE:2;
            uint8_t BORINT:1;
            uint8_t BORRST:1;
            uint8_t RESERVED:3;
            uint8_t BOREN:1;
        };
        uint8_t BYTE;
    } PMUBOR;
    uint8_t RESERVED0;
    union {
        struct {
            uint8_t EXP:4;
            uint8_t MANT:4;
        };
        uint8_t BYTE;
    } WKPTIME;
    union {
        struct {
            uint8_t RTSELB:3;
            uint8_t RTSELA:3;
            uint8_t BATOP:1;
            uint8_t SERFAST:1;
        };
        uint8_t BYTE;
    } PMUCONF;
} PMU_SFRS_t;

/**
 * @brief The starting address of PMU SFRS.
 */
#define PMU_SFRS              ((__IO PMU_SFRS_t *) (SDIO_BASE + 0x1))

#define E_PMU_PMUBOR_BOUTVALUE_2V0      0
#define E_PMU_PMUBOR_BOUTVALUE_2V2      1
#define E_PMU_PMUBOR_BOUTVALUE_2V4      2
#define E_PMU_PMUBOR_BOUTVALUE_2V6      3

#define E_PMU_POWER_MODE_LDO            0
#define E_PMU_POWER_MODE_BATTERY        1

/**
 * @brief Set wake up timer.
 *
 * @param mantissa The mantissa value of wakeup timer setting.
 * @param exponent exponential value of wakeup timer setting.
 */
static __INLINE void f_PMU_WakeupTime(uint8_t mantissa, uint8_t exponent)
{
    PMU_SFRS->WKPTIME.MANT = mantissa;
    PMU_SFRS->WKPTIME.EXP = exponent;
}

/**
 * @brief Force a hardware reset.
 */
static __INLINE void f_PMU_HwReset( void )
{
    PMU_SFRS->PMURST.HWRST = 1;
}

/**
 * @brief Force a software reset.
 */
static __INLINE void f_PMU_SwReset( void )
{
    PMU_SFRS->PMURST.SWRST = 1;
}

/**
 * @brief Put system in sleep mode and then continue at wakeup.
 */
static __INLINE void f_PMU_SleepContinue(void)
{
    PMU_SFRS->PMURST.DLEEP = 1;
}

/**
 * @brief Put system in sleep mode and then reboot at wakeup.
 */
static __INLINE void f_PMU_SleepReboot(void)
{
    PMU_SFRS->PMURST.DLEEP = 1;
}

/**
 * @brief Set system power mode.
 *
 * @param mode The remuneration of power mode.
 */
static __INLINE void f_PMU_SetPowerMode(uint8_t mode)
{
    PMU_SFRS->PMUCONF.BATOP = mode;
}

/**
 * @brief Enable brownout.
 */
static __INLINE void f_PMU_EnableBrownOut(void)
{
    PMU_SFRS->PMUBOR.BOREN = 1;
}

/**
 * @brief Disable brownout.
 */
static __INLINE void f_PMU_DisableBrownOut(void)
{
    PMU_SFRS->PMUBOR.BOREN = 0;
}

/**
 * @brief Enable brownout reset.
 */
static __INLINE void f_PMU_EnableBrownOutReset(void)
{
    PMU_SFRS->PMUBOR.BORRST = 1;
}

/**
 * @brief Disable brownout reset.
 */
static __INLINE void f_PMU_DisableBrownOutReset(void)
{
    PMU_SFRS->PMUBOR.BORRST = 0;
}

/**
 * @brief Enable brownout interrupt.
 */
static __INLINE void f_PMU_EnableBrownOutInterrupt(void)
{
    PMU_SFRS->PMUBOR.BORINT = 1;
}

/**
 * @brief Disable brownout interrupt.
 */
static __INLINE void f_PMU_DisableBrownOutInterrupt(void)
{
    PMU_SFRS->PMUBOR.BORINT = 0;
}

/**
 * @brief Set brown out voltage level.
 *
 * @param level The enumeration of brownout voltage level.
 */
static __INLINE void f_PMU_SetBrownOutLevel(uint8_t level)
{
    PMU_SFRS->PMUBOR.BOUTVALUE = level;
}

#endif /* __PMU_SFRS_H__ */
