/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file rtc_sfrs.h
 */
 
#ifndef __RTC_SFRS_H__
#define __RTC_SFRS_H__

#define E_RTC_MAX_COUNT  0x0003FFFF

/**
 * @brief A structure to represent Special Function Registers for RTC.
 */
typedef struct {
    struct {
        uint32_t LOCK:1;
        uint32_t EN:1;
        uint32_t OWIE:1;
        uint32_t ALRIE:1;
        uint32_t OWF:1;
        uint32_t ALRF0:1;
        uint32_t ALRF1:1;
        uint32_t :1;
    } CTRL;
    struct {
        uint32_t CNT:18;
        uint32_t :14;
    } COUNTER;
    struct {
        uint32_t CNT:18;
        uint32_t :14;
    } ALARM0;
    struct {
        uint32_t CNT:18;
        uint32_t :14;
    } ALARM1;
} RTC_SFRS_t;

/**
 * @brief The starting address of RTC SFRS.
 */
#define RTC_SFRS              ((__IO RTC_SFRS_t *) (0x50000040))

/**
 * @brief A structure to represent Special Function Registers for RTC trim.
 */
typedef struct {
    uint8_t TRIM:2;
    uint8_t :6;
} RTC_TRIM_SFRS_t;

/**
 * @brief The starting address of RTC TRIM SFRS.
 */
#define RTC_TRIM_SFRS           ((__IO RTC_TRIM_SFRS_t *) (0x5001800B))

/**
 * @brief Enable RTC timers.
 */
static __INLINE void f_RTC_Enable(void)
{
    RTC_SFRS->CTRL.EN = 1;
}

/**
 * @brief Disable RTC timers.
 */
static __INLINE void f_RTC_Disable(void)
{
    RTC_SFRS->CTRL.EN = 0;
}

/**
 * @brief Unlock RTC configuration mode.
 */
static __INLINE void f_RTC_Unlock(void)
{
    RTC_SFRS->CTRL.LOCK = 0;
}

/**
 * @brief Lock RTC configuration mode.
 */
static __INLINE void f_RTC_Lock(void)
{
    RTC_SFRS->CTRL.LOCK = 1;
}

/**
 * @brief Enable RTC alarm interrupt.
 */
static __INLINE void f_RTC_EnableAlarmIRQ(void)
{
    RTC_SFRS->CTRL.ALRIE = 1;
}

/**
 * @brief Disable RTC alarm interrupt.
 */
static __INLINE void f_RTC_DisableAlarmIRQ(void)
{
    RTC_SFRS->CTRL.ALRIE = 0;
}

/**
 * @brief Enable RTC overflow interrupt.
 */
static __INLINE void f_RTC_EnableOverflowIRQ(void)
{
    RTC_SFRS->CTRL.OWIE = 1;
}

/**
 * @brief Disable RTC overflow interrupt.
 */
static __INLINE void f_RTC_DisableOverflowIRQ(void)
{
    RTC_SFRS->CTRL.OWIE = 0;
}

/**
 * @brief Check if RTC overflow occured.
 */
static __INLINE uint8_t f_RTC_IsOverflow(void)
{
    return RTC_SFRS->CTRL.OWF;
}

/**
 * @brief Check if RTC alarm0 detected.
 */
static __INLINE uint8_t f_RTC_IsAlarm0(void)
{
    return RTC_SFRS->CTRL.ALRF0;
}

/**
 * @brief Check if RTC alarm1 detected.
 */
static __INLINE uint8_t f_RTC_IsAlarm1(void)
{
    return RTC_SFRS->CTRL.ALRF1;
}

/**
 * @brief Read RTC current counter.
 */
static __INLINE uint32_t f_RTC_GetCurrentCounter(void)
{
    return RTC_SFRS->COUNTER.CNT;
}

/**
 * @brief Set RTC current counter.
 */
static __INLINE void f_RTC_SetCurrentCounter(uint32_t val)
{
    RTC_SFRS->COUNTER.CNT = val;
}

/**
 * @brief Read RTC alarm 0 counter.
 */
static __INLINE uint32_t f_RTC_GetAlarm0Counter(void)
{
    return RTC_SFRS->ALARM0.CNT;
}

/**
 * @brief Set RTC alarm 0 counter.
 */
static __INLINE void f_RTC_SetAlarm0Counter(uint32_t val)
{
    RTC_SFRS->ALARM0.CNT = val;
}

/**
 * @brief Read RTC alarm 1 counter.
 */
static __INLINE uint32_t f_RTC_GetAlarm1Counter(void)
{
    return RTC_SFRS->ALARM1.CNT;
}

/**
 * @brief Set RTC alarm 1 counter.
 */
static __INLINE void f_RTC_SetAlarm1Counter(uint32_t val)
{
    RTC_SFRS->ALARM1.CNT = val;
}

/**
 * @brief Set RTC trim setting.
 */
static __INLINE void f_RTC_SetRTCTrim(uint8_t val)
{
    RTC_TRIM_SFRS->TRIM = val;
}

#endif /* __RTC_SFRS_H__ */