/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file spi_sfrs.h
 */

#ifndef __SPI_SFRS_H__
#define __SPI_SFRS_H__

/**
 * @brief A structure to represent Special Function Registers for SPI.
 */
typedef struct {
    union {
        struct {
            uint8_t SCKSTD:2;
            uint8_t CPHA:1;
            uint8_t CPOL:1;
            uint8_t MSTR:1;
            uint8_t Reserved:2;
            uint8_t SINTE:1;
        };
        uint8_t SPCR;    
    } SPCR;
    union {
        struct {
            uint8_t SRXFE:1;
            uint8_t SRXFF:1;
            uint8_t STXFE:1;
            uint8_t STXFF:1;
            uint8_t Reserved:2;
            uint8_t SWCOL:1;
            uint8_t SINTF:1;
        };
        uint8_t SPSR;
    } SPSR;
    uint8_t  SPDR;
    union {
        struct {
            uint8_t SCKEXT:2;
            uint8_t SPE:1;
            uint8_t Reserved:3;
            uint8_t SICNT:2;
        };
        uint8_t SPER;
    } SPER;
} SPI_SFRS_t;

/**
 * @brief The starting address of SPI SFRS.
 */
#define SPI_SFRS              ((__IO SPI_SFRS_t *) (0x5000001C))

#define E_SPI_CLOCK_DIV2                0
#define E_SPI_CLOCK_DIV4                1
#define E_SPI_CLOCK_DIV8                0
#define E_SPI_CLOCK_DIV16               2
#define E_SPI_CLOCK_DIV32               3
#define E_SPI_CLOCK_DIV64               1
#define E_SPI_CLOCK_DIV128              2
#define E_SPI_CLOCK_DIV256              3
#define E_SPI_CLOCK_DIV512              0
#define E_SPI_CLOCK_DIV1024             1
#define E_SPI_CLOCK_DIV2048             2
#define E_SPI_CLOCK_DIV4096             3

#define E_SPI_CLOCK_EXT_DIV2            0
#define E_SPI_CLOCK_EXT_DIV4            0
#define E_SPI_CLOCK_EXT_DIV8            1
#define E_SPI_CLOCK_EXT_DIV16           0
#define E_SPI_CLOCK_EXT_DIV32           0
#define E_SPI_CLOCK_EXT_DIV64           1
#define E_SPI_CLOCK_EXT_DIV128          1
#define E_SPI_CLOCK_EXT_DIV256          1
#define E_SPI_CLOCK_EXT_DIV512          2
#define E_SPI_CLOCK_EXT_DIV1024         2
#define E_SPI_CLOCK_EXT_DIV2048         2
#define E_SPI_CLOCK_EXT_DIV4096         2

#define E_SPI_CLOCK_PHASE0              0
#define E_SPI_CLOCK_PHASE1              1

#define E_SPI_CLOCK_POLARITY0           0
#define E_SPI_CLOCK_POLARITY1           1

#define E_SPI_STATUS_INTERRUPT          ()uint8_t) (1<<7))
#define E_SPI_STATUS_WRITE_COLLISIION   ((uint8_t) (1<<6))
#define E_SPI_STATUS_TX_FIFO_FULL       ((uint8_t) (1<<3))
#define E_SPI_STATUS_TX_FIFO_EMPTY      ((uint8_t) (1<<2))
#define E_SPI_STATUS_RX_FIFO_FULL       ((uint8_t) (1<<1))
#define E_SPI_STATUS_RX_FIFO_EMPTY      ((uint8_t) (1<<0))

#define E_SPI_INT_DELAY_COUNT0          0
#define E_SPI_INT_DELAY_COUNT1          1
#define E_SPI_INT_DELAY_COUNT2          2
#define E_SPI_INT_DELAY_COUNT3          3

/**
 * @brief Set SPI clock divider
 *
 * @param div The setting for standard divider bits
 * @param ext_div The setting for extended divider bits
 */
static __INLINE void f_SPI_ClockDiv(uint8_t div, uint8_t ext_div)
{
    SPI_SFRS->SPCR.SCKSTD = div;
    SPI_SFRS->SPER.SCKEXT = ext_div;
}

/**
 * @brief Set SPI phase
 */
static __INLINE void f_SPI_SetPhase(uint8_t phase)
{
    SPI_SFRS->SPCR.CPHA = phase;
}

/**
 * @brief Set SPI Polarity
 */
static __INLINE void f_SPI_SetPolarity(uint8_t Polarity)
{
    SPI_SFRS->SPCR.CPOL = Polarity;
}

/**
 * @brief Set SPI mode
 */
static __INLINE void f_SPI_SetMode(uint8_t mode)
{
    SPI_SFRS->SPCR.MSTR = mode;
}

/**
 * @brief Set SPI interrupt delay cycles
 */
static __INLINE void f_SPI_IRQDelay(uint8_t cnt)
{
    SPI_SFRS->SPER.SICNT = cnt;
}

/**
 * @brief Enable SPI hardware module
 */
static __INLINE void f_SPI_Enable(void)
{
    SPI_SFRS->SPER.SPE = 1;
}

/**
 * @brief Disable SPI hardware module
 */
static __INLINE void f_SPI_Disable(void)
{
    SPI_SFRS->SPER.SPE = 0;
}

/**
 * @brief Enable SPI interrupt
 */
static __INLINE void f_SPI_IRQEnable(void)
{
    SPI_SFRS->SPCR.SINTE = 1;
}

/**
 * @brief Disable SPI interrupt
 */
static __INLINE void f_SPI_IRQDisable(void)
{
    SPI_SFRS->SPCR.SINTE = 0;
}

/**
 * @brief Read a byte from SPI data register
 */
static __INLINE uint8_t f_SPI_ReadData(void)
{
    return SPI_SFRS->SPDR;
}

/**
 * @brief Write a byte to SPI data register
 */
static __INLINE void f_SPI_WriteData(uint8_t data)
{
    SPI_SFRS->SPDR = data;
}

/**
 * @brief Get SPI status
 */
static __INLINE uint8_t f_SPI_GetStatus(void)
{
    return SPI_SFRS->SPSR.SPSR;
}

/**
 * @brief Clear SPI interrupt flag
 */
static __INLINE void f_SPI_ClearIRQ(void)
{
    SPI_SFRS->SPSR.SINTF = 1;
}

/**
 * @brief Check if transnmitter FIFO buffer full
 *
 * @return The status bit for transnmitter FIFO buffer full.
 */
static __INLINE uint8_t f_SPI_TxFIFOFull(void)
{
    return SPI_SFRS->SPSR.STXFF;
}

/**
 * @brief Check if transnmitter FIFO buffer empty
 *
 * @return The status bit for transnmitter FIFO buffer empty.
 */
static __INLINE uint8_t f_SPI_TxFIFOEmpty(void)
{
    return SPI_SFRS->SPSR.STXFE;
}

/**
 * @brief Check if receiver FIFO buffer full
 *
 * @return The status bit for receiver FIFO buffer full.
 */
static __INLINE uint8_t f_SPI_RxFIFOFull(void)
{
    return SPI_SFRS->SPSR.SRXFF;
}

/**
 * @brief Check if receiver FIFO buffer empty
 *
 * @return The status bit for receiver FIFO buffer empty.
 */
static __INLINE uint8_t f_SPI_RxFIFOEmpty(void)
{
    return SPI_SFRS->SPSR.SRXFE;
}

#endif /* __SPI_SFRS_H__ */