/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file tx_sfrs.h
 */

#ifndef __TX_SFRS_H__
#define __TX_SFRS_H__

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// RF Transmitter Definition
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//The procedures of RF transmission are as follows:
//1. Select ASK mode (for Krankl it only supports ASK mode)
//2. Set NX, NF for PLL to match 433.92MHZ
//3. Given burst bit number, wait bit number, chip rate 
//4. Set timing of RF such as TPATCH, TED
//5, put first two bytes data into buffer and start TX
//6. TX will be interrupt by reload or burst done

#define TXCTRL0_MODUL_ASK ((uint8_t) (0x03 << 4))
//#define TXCTRL0_MODUL_PSK ((uint8_t) (0x01 << 4))
//#define TXCTRL0_MODUL_FSK ((uint8_t) (0x02 << 4))
#define TXCTRL0_FRAC_EN   ((uint8_t) (0x01 << 3))
#define TXCTRL0_NXTBURST  ((uint8_t) (0x01 << 6))
#define TXCTRL0_STARTX    ((uint8_t) (0x01 << 7))
#define TXCTRL1_PEDES_10 ((uint8_t) (0x00 << 6))
#define TXCTRL1_PEDES_12 ((uint8_t) (0x01 << 6))
#define TXCTRL1_PEDES_14 ((uint8_t) (0x02 << 6))
#define TXCTRL1_PEDES_16 ((uint8_t) (0x03 << 6))

#define TXTRIM0_VAL ((uint8_t) 0x44)
#define TXTRIM1_VAL ((uint8_t) 0x82)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// RF Transmitter Structure
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/// RFTX_CTRL: RF transmitter configuration
/// STARTX: initiate an RF burst
///   1 => start a transmission burst with the current settings
///        use: TXCTRL0_STARTX
///
/// NXTBURST: indicate to repeat the current burst again once the current one
/// completes.   This bit is checked at the end of each rf transmission burst.
/// if set, then another burst is initiated automatically without needing to
/// set STARTX again.  This bit can be adjusted during transmission.
///   0 => transmitter will become idle after current burst
///   1 => transmitter will start new burst after current one.
///        use: TXCTRL0_NXTBURST
///
/// MODUL: modulation format to use
///   00 => reserved
///   01 => BPSK            use: TXCTRL0_MODUL_PSK
///   10 => BFSK            use: TXCTRL0_MODUL_FSK
///   11 => ASK             use: TXCTRL0_MODUL_ASK
///
/// FRAC_EN: enable fractional-N feature of PLL
///   0 => integer-N PLL mode
///   1 => fractional-N PLL mode   use: TXCTRL0_FRAC_EN
///
/// TXCTRL0[2:0]: reserved.  only write 3'b000
///
/// PEDES: Controls power level set during TDET3.  
///   00 => 0x10   use: TXCTRL1_PEDES_10
///   01 => 0x12   use: TXCTRL1_PEDES_12
///   10 => 0x14   use: TXCTRL1_PEDES_14
///   11 => 0x16   use: TXCTRL1_PEDES_16
///
/// TPATCH[10:5]:  Allows fine tuning of the time between message bursts
/// when NXTBURST is set.  Since the NWAIT parameter only allows integer
/// number so of chip times to be set for the inter-burst waiting period
/// this register can be used to add a small delay with finer resolution
/// to produce the desired time.  The approximate time resolution for
/// this variable is 800ns and is unaffected by CKDIV.  Note that only
/// the MSBs of this parameter are stored in this register.  The MSBs are
/// stored in the TXTIMING register.
///
/// POWLEV:  Power level for FSK, and for logic level 0 of ASK.  When using
/// FSK modulation, the output power os controlled by this register.  When
/// using ASK modulation, the power level output during the low level chips
/// is controlled by this register.  To do on-off keying ASK, set this to
/// 0.  Ignored in BPSK mode.
///
/// ASKAMP:  Power level for BPSK, and for logic level 1 of ASK.  Ignored
/// in FSK mode.
///
typedef struct
{
  __IO uint8_t TXCTRL0;  // { STARTX , NXTBURST , MODUL[1:0] , FRAC_EN , RESERVED[2:0] }
  __IO uint8_t TXCTRL1;  // { PEDES[1:0] , TPATCH[10:5] }
  __IO uint8_t POWLEV;   // { POWLEV[7:0] }
  __IO uint8_t POWAP;    // { ASKAMP[7:0] }
} RFTX_CTRLType;



/// TXDAT: Transmit data buffer.  This buffer must be periodically written to
/// with transmit data during the rf transmission.  Before the transmission
/// begins, it must contain the first 16 chips to be transmitted.  After
/// approximately 8 chips have been transmitted, a TX_RELOAD_IRQn will be
/// generated indicating the buffer should be written to with the next
/// 16 chips.  Chips are transmitted out of the buffer MSB first.  If the
/// number of chips to be transmitted is not a multiple of 16, then the
/// most-significant bits of the last buffer update are the ones transmitted. 
typedef union
{
  __IO uint16_t HWORD;
  __IO uint8_t  BYTE[2];
} RFTX_DATAType;


/// RFTX_MSGFMT: Transmit message format control
/// NWAIT: Number of chip periods to wait between transmission bursts.
/// This register can be adjusted for the *NEXT* inter-burst period after
/// an inter-burst period commences without affecting the number of chips
/// periods for the current inter-burst period.
///
/// NXBITS: Number of tranmsission chips in the burst.  This register can
/// be adjusted for the *NEXT* burst after a burst commences without
/// affecting the number of chips transmitted during that burst.
///
typedef union
{
  __IO uint16_t HWORD; // { NWAIT[5:0] , NXBITS[9:0] }
  struct{
    __IO uint8_t NBURST;   // { NXBITS[7:0] }
    __IO uint8_t TXCTRL2;  // { NWAIT[5:0] , NXBITS[9:8] }
  }BYTE;                        
} RFTX_MSGFMTType;



/// RFTX_TIMING: TX timing control
///
/// TPATCH[4:0]: see description in TXCONFIG section
///
/// TXCLKDIV: controls chip rate for the rf message.  This number
/// multiplied by approximately 800ns to determine the duration of
/// each transmitted chip.
///
/// TDET1: controls time between analog bias enable and PLL enable
/// (bias settling time).
///   0 => 1.6 us  (recommended)
///   1 => 8.0 us
///
/// TDET2: controls time between PLL enable and PA enable (PLL settling time).
/// The delay is (1+TDET2*1024)*800ns.  Recommended value is 3'b010.
///
/// TDET3: controls the time between PA enable and power ramp up (pedestal
/// time).  The delay is (1+TDET3*1024)*800ns.  Recommended value is 3'b101.
///
/// TDET4: controls the time between power ramp up and initiation of modulation
/// (power level settling).  The delay is (1+TDET4*1024)*800ns.  Recommended
/// value is 5'b10100.
///
typedef union
{
  __IO uint16_t HWORD[2];   // HWORD[1]=> { TDET3[2:0] , TDET4[4:0],  TRAMP[3:0] , TDET2[2:0] , TDET1}								
                            //HWORD[0]=> { TPATCH[4:0] , TXCLKDIV[10:0]} 
  struct{
    __IO uint8_t TXCLKDIV;     // {TXCLKDIV[7:0] }
    __IO uint8_t TXCTRL3;   //{ TPATCH[4:0] , TXCLKDIV[10:8] }
    __IO uint8_t TXCTRL4;    // { TRAMP[3:0] , TDET2[2:0] , TDET1}
    __IO uint8_t TXCTRL5;  //{ TDET3[2:0] , TDET4[4:0] }
  }BYTE;

} RFTX_TIMINGType;



/// RFTX_PLLCTRL:  PLL frequency control
///
/// NF , NX:  Control the frequency multiplication ratio of the PLL.  The RF
/// transmit frequency will by fXTL * a multiplication ratio which is:
///   If FRAC_EN is 1 then:  8 + nx                  (integer-N mode)
///   If FRAC_EN is 1 hen:   9 + nx + nf/(2^14)      (fractional-N mode)
///
/// DF:  Frequency deviation of the logic 1 level in FSM mode.  This number is
/// added to NF during the logic 1 chips in FSK mode.  Ignored in other modes.
///
/// LPFTRIM:  Loop filter trim for PLL.  Recommended value is 4'b0011.
///
/// CPTRIM:  Charge pump trim for PLL.  Recommended value is 3'b001.
///
typedef union
{
  __IO uint32_t WORD; // {DF[5:0], LPFTRIM[3:0], CPTRIM[2:0], NX[4:0], NF[13:0]}
  __IO uint16_t HWORD[2]; // HWORD[1] ==> { DF[5:0] , LPFTRIM[3:0] , CPTRIM[2:0] , NX[4:2] }
                           // HWORD[0] ==> { NX[1:0] , NF[13:0] }
  struct {
    __IO uint8_t PLLCTRL0;  // { NF[7:0] }	 
    __IO uint8_t PLLCTRL1;  // { NX[1:0] , NF[13:8] }
    __IO uint8_t PLLCTRL2;   // { LPFTRIM[1:0] , CPTRIM[2:0] , NX[4:2] }
    __IO uint8_t PLLCTRL3; 	// { DF[5:0] , LPFTRIM[3:2] }				              
  }BYTE;                         					   
}  RFTX_PLLType;   

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// RF Transmitter Related Registers 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define TX_BASE (ASIC_7B_BASE + 0x32)

#define RFTX_CTRL ((RFTX_CTRLType *) (ASIC_7B_BASE + 0x32))

#define RFTX_DATA ((RFTX_DATAType *) (ASIC_7B_BASE + 0x36))

#define RFTX_MSGFMT ((RFTX_MSGFMTType *) (ASIC_7B_BASE + 0x38))


#define RFTX_TIMING ((RFTX_TIMINGType *) (ASIC_7B_BASE + 0x3A))


#define RFTX_PLL ((RFTX_PLLType *) (ASIC_16B_BASE + 0x8000))


/// TXTRIM0 - reserved, only ever write 0x44
/// TXTRIM1 - reserved, only ever write 0x82
#define TXTRIM0     ((__IO uint8_t *) (ASIC_16B_BASE + 0x03))

#define TXTRIM1     ((__IO uint8_t *) (ASIC_16B_BASE + 0x0C))



///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// RF Transmitter Function
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

static __INLINE void TX_StartTransmission()
{
    RFTX_CTRL->TXCTRL0 |= TXCTRL0_STARTX;
    
}

static __INLINE void TX_StopTransmission()
{
    RFTX_CTRL->TXCTRL0 &= ~TXCTRL0_STARTX;
}

static __INLINE void TX_En_NextBurst()
{
    RFTX_CTRL->TXCTRL0 |= TXCTRL0_NXTBURST;	
}

static __INLINE void TX_Dis_NextBurst()
{
    RFTX_CTRL->TXCTRL0 &= ~TXCTRL0_NXTBURST;
} 

/**
* @brief RF TX Control 
*
* @param module: TXCTRL0_MODUL_ASK = select ASK module
* @param frac_en: TXCTRL0_FRAC_EN = enable fractionall PLL
* @param peds: pedestral time
* TXCTRL1_PEDES_10  
* TXCTRL1_PEDES_12
* TXCTRL1_PEDES_14
* TXCTRL1_PEDES_16
*/

static __INLINE void TX_TXconfig(uint8_t module, uint8_t frac_en, uint8_t peds)
{
    RFTX_CTRL->TXCTRL0 = (module | frac_en);
    RFTX_CTRL->TXCTRL1 |= peds; 
}

/**
* @brief RF TX amplifier power
*
* @param powlev: 0 - 127 
* @param powap:  0 - 127
*/
static __INLINE void TX_SetPowerLevel(uint8_t powlev, uint8_t powap)
{
    RFTX_CTRL->POWLEV = powlev;
    RFTX_CTRL->POWAP = powap;
}
/**
* @brief RF TX set patch time 
* @param tpatch: patch time 
*/
static __INLINE void TX_SetTpatch(uint16_t tpatch)
{
    RFTX_CTRL->TXCTRL1 |= ((uint8_t)(tpatch>>5) & 0x3F);
    RFTX_TIMING->BYTE.TXCTRL3 |= ((uint8_t)(tpatch << 3));	
}

/**
* @brief RF TX set message format including burst bit and wait bit time
* @param nburst: 0 - 1023 
* @param nwait:  0 - 63 
*/
static __INLINE void TX_SetMsgFormat(uint16_t nburst, uint8_t nwait)
{
    RFTX_MSGFMT->HWORD = (nwait<<10 & 0xFC00)|(nburst & 0x03FF);
}   

/**
* @brief RF TX clock divider number 
* chip rate = 1.25Mbps / TXCLKDIV
* for example, if chip rate is 3.3K, TXCDIV = 375
* @param txcdiv: 0 - 2047 
*/
static __INLINE void TX_SetChipRate(uint16_t txcdiv)
{
    RFTX_TIMING->HWORD[0] |= (0x07FF & txcdiv);	
}

/**
* @brief: given the number of wait bit time and using default TED value to calculate
* the value of NWAIT[5:0] and TPATCH[10:0] 
* The details of calculation is given in the data sheet
* The default TED1 is set 1'b0, TED2 3'b010, TED3 3'b101, TED4 = TED5 = 5'b10100
* @param txcdiv: 0 - 2047
* @param nburst: 0 - 1023 
* @param nwait_total:  0 - 63 (here nwait is total bit time between two bursts)
*/
static void TX_SetTiming(uint16_t txcdiv, uint16_t nburst, uint8_t nwait_total)
{  
    uint8_t NWAIT = (uint8_t)(nwait_total - 340.0 / (4 * txcdiv));
    uint16_t TPATCH = (uint16_t)(((nwait_total - NWAIT) * txcdiv *4/5 - 68 - 0.8 ) / 0.781);
    TX_SetTpatch(TPATCH);
    TX_SetMsgFormat(nburst, NWAIT);
    TX_SetChipRate(txcdiv);
    RFTX_TIMING->HWORD[1] = 0xB404; //default TED value 
}

/**
* @brief: Set TX frequency
* The default value is for RF Freq: 433.92Mhz at XTAL Freq 30MHz.
*/
static __INLINE void TX_SetFreq()
{
    RFTX_PLL->WORD = 0x00C95E04;
}

static __INLINE void TX_LoadData(uint16_t data)
{
    RFTX_DATA->HWORD = data;
}

#endif /* __TX_SFRS_H__ */