/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file uart_sfrs.h
 */

#ifndef __UART_SFRS_H__
#define __UART_SFRS_H__

/**
 * @brief A structure to represent Special Function Registers for UART.
 */
typedef struct {
    uint8_t DATA;
    union {
        struct {
            uint8_t URXIEN:1;
            uint8_t UTXIEN:1;
            uint8_t URXERREN:1;
            uint8_t UTOUTIEN:1;
            uint8_t UISTTS:4;
        };
        struct {
            uint8_t INTEN:4;
            uint8_t :4;
        };
        uint8_t BYTE;
    } ICTCL;
    union {
        struct {
            uint8_t USIZE:2;
            uint8_t USTOP:1;
            uint8_t UPAREN:1;
            uint8_t UPARITY:1;
            uint8_t USTICKEN:1;
            uint8_t UBREAKEN:1;
            uint8_t ULOOPEN:1;
        };
        uint8_t BYTE;
    } LCTRL;
    union {
        struct {
            uint8_t :3;
            uint8_t UARTEN:1;
            uint8_t :4;
        };
        uint8_t BYTE;
    } CTRL1;
    union {
        struct {
            uint8_t UDTRDY:1;
            uint8_t UOVRUNERR:1;
            uint8_t UPRTYERR:1;
            uint8_t UFRMERR:1;
            uint8_t UBREAKINT:1;
            uint8_t UTXFFEMPTY:1;
            uint8_t UTXEMPTY:1;
            uint8_t UERR:1;
        };
        uint8_t BYTE;
    } STATUS;
    uint16_t DIV;
} UART_SFRS_t;

#define UART_SFRS ((__IO UART_SFRS_t *) (0x50000010)) /*!< The starting address of UART SFRS. */

#define E_UART_STATUS_DATA_READY        ((uint8_t) (1<<0))  /*!< UART data ready. */
#define E_UART_STATUS_OVERRUN_ERROR     ((uint8_t) (1<<1))  /*!< UART over run error occurred. */
#define E_UART_STATUS_PARITY_ERROR      ((uint8_t) (1<<2))  /*!< UART parity error occurred. */
#define E_UART_STATUS_FRAME_ERROR       ((uint8_t) (1<<3))  /*!< UART frame error occurred. */
#define E_UART_STATUS_BREAKINT_ERROR    ((uint8_t) (1<<4))  /*!< UART break interrupt occurred. */
#define E_UART_STATUS_TX_FIFO_EMPTY     ((uint8_t) (1<<5))  /*!< UART transmission FIFO empty. */
#define E_UART_STATUS_TX_EMPTY          ((uint8_t) (1<<6))  /*!< UART transmitter empty. */
#define E_UART_STATUS_ERROR             ((uint8_t) (1<<7))  /*!< UART error occurred. */

#define E_UART_INT_STATUS_NONE          0x01    /*!< UART no interrupt. */
#define E_UART_INT_STATUS_TX_COMPLETE   0x02    /*!< UART transmission complete. */
#define E_UART_INT_STATUS_RX_READY      0x04    /*!< UART data received. */
#define E_UART_INT_STATUS_RX_ERROR      0x06    /*!< UART error occurred in receiver. */
#define E_UART_INT_STATUS_RX_TIMEOUT    0x0C    /*!< UART receiver time out. */

          
#define E_UART_DATA_SIZE_5BIT   0       /*!< UART data size 5 bits. */
#define E_UART_DATA_SIZE_6BIT   1       /*!< UART data size 6 bits. */
#define E_UART_DATA_SIZE_7BIT   2       /*!< UART data size 7 bits. */
#define E_UART_DATA_SIZE_8BIT   3       /*!< UART data size 8 bits. */

#define E_UART_STOP_BIT_1       0       /*!< UART 1 stop bit. */
#define E_UART_STOP_BIT_2       1       /*!< UART 2 stop bit. */

#define E_UART_PARITY_BIT_ODD   0       /*!< UART odd parity bit. */
#define E_UART_PARITY_BIT_EVEN  1       /*!< UART even parity bit. */

#define E_UART_INT_DELAY_1      0       /*!< UART interrupt delay 1 cycle. */
#define E_UART_INT_DELAY_3      1       /*!< UART interrupt delay 3 cycles. */

/**
 * @brief Set UART baud rate divider.
 *
 * @param div The baud rate divider.
 */
static __INLINE void f_UART_BaudRateDiv(uint32_t div)
{
    UART_SFRS->DIV = div;
}

/**
 * @brief Get UART line status.
 *
 * @return The line status byte.
 */
static __INLINE uint8_t f_UART_LineStatus(void)
{
    return UART_SFRS->STATUS.BYTE;
}

/**
 * @brief Reset UART receiver FIFO.
 */
static __INLINE void f_UART_RxFiFoReset(void)
{
}

/**
 * @brief Reset UART transmitter FIFO.
 */
static __INLINE void f_UART_TxFiFoReset(void)
{
}

/**
 * @brief Enable UART hardware module.
 */
static __INLINE void f_UART_Enable(void)
{
    UART_SFRS->CTRL1.UARTEN = 1;
}

/**
 * @brief Disable UART hardware module.
 */
static __INLINE void f_UART_Disable(void)
{
    UART_SFRS->CTRL1.UARTEN = 0;
}

/**
 * @brief UART receive data
 *
 * @return The data received.
 */
static __INLINE uint8_t f_UART_RxData(void)
{
    return UART_SFRS->DATA;
}

/**
 * @brief UART transmit data
 *
 * @param data The data to be transmitted.
 */
static __INLINE void f_UART_TxData(uint8_t data)
{
    UART_SFRS->DATA = data;
}

/**
 * @brief Enable UART receiver interrupt
 */
static __INLINE void f_UART_EnableRxIRQ(void)
{
    UART_SFRS->ICTCL.URXIEN = 1;
}

/**
 * @brief Disable UART receiver interrupt
 */
static __INLINE void f_UART_DisableRxIRQ(void)
{
    UART_SFRS->ICTCL.URXIEN = 0;
}

/**
 * @brief Enable UART transmitter interrupt
 */
static __INLINE void f_UART_EnableTxIRQ(void)
{
    UART_SFRS->ICTCL.UTXIEN = 1;
}

/**
 * @brief Disable UART transmitter interrupt
 */
static __INLINE void f_UART_DisableTxIRQ(void)
{
    UART_SFRS->ICTCL.UTXIEN = 0;
}

/**
 * @brief Enable UART receiver time out interrupt
 */
static __INLINE void f_UART_EnableRxTimeoutIRQ(void)
{
    UART_SFRS->ICTCL.UTOUTIEN = 1;
}

/**
 * @brief Disable UART receiver time out interrupt
 */
static __INLINE void f_UART_DisableRxTimeoutIRQ(void)
{
    UART_SFRS->ICTCL.UTOUTIEN = 0;
}

/**
 * @brief Enable UART receiver error interrupt
 */
static __INLINE void f_UART_EnableRxErrorIRQ(void)
{
    UART_SFRS->ICTCL.URXERREN = 1;
}

/**
 * @brief Disable UART receiver error interrupt
 */
static __INLINE void f_UART_DisableRxErrorIRQ(void)
{
    UART_SFRS->ICTCL.URXERREN = 0;
}

/**
 * @brief Check if UART transmitter empty
 *
 * @return The status of UART empty flag.
 */
static __INLINE uint8_t f_UART_TxEmpty(void)
{
    return UART_SFRS->STATUS.UTXEMPTY;
}

/**
 * @brief check if UART data received
 *
 * @return The status of UART data ready flag.
 */
static __INLINE uint8_t f_UART_RxDataReady(void)
{
    return UART_SFRS->STATUS.UDTRDY;
}

/**
 * @brief Get UART interrupt status
 *
 * @return The value of UART interrupt status register.
 */
static __INLINE uint8_t f_UART_IntStatus(void)
{
    return UART_SFRS->ICTCL.UISTTS;
}

/**
 * @brief Set UART data size in number of bits
 *
 * @param size The number of bits.
 */
static __INLINE void f_UART_SetDataSize(uint8_t size)
{
    switch (size) {
    case 8:
        UART_SFRS->LCTRL.USIZE = E_UART_DATA_SIZE_8BIT;
        break;
    case 7:
        UART_SFRS->LCTRL.USIZE = E_UART_DATA_SIZE_7BIT;
        break;
    case 6:
        UART_SFRS->LCTRL.USIZE = E_UART_DATA_SIZE_6BIT;
        break;
    case 5:
        UART_SFRS->LCTRL.USIZE = E_UART_DATA_SIZE_5BIT;
        break;
    default:
        UART_SFRS->LCTRL.USIZE = E_UART_DATA_SIZE_8BIT;
        break;
    }
}

/**
 * @brief Set UART stop bit size in number of bits
 *
 * @param stop The number of bits.
 */
static __INLINE void f_UART_SetStopBit(uint8_t stop)
{
    switch (stop) {
    case 1:
        UART_SFRS->LCTRL.USTOP = E_UART_STOP_BIT_1;
        break;
    default:
        UART_SFRS->LCTRL.USTOP = E_UART_STOP_BIT_2;
        break;
    }
}

/**
 * @brief Set UART parity bit as even
 */
static __INLINE void f_UART_SetEvenParityBit(void)
{
    UART_SFRS->LCTRL.UPARITY = E_UART_PARITY_BIT_EVEN;
}

/**
 * @brief Set UART parity bit as odd
 */
static __INLINE void f_UART_SetOddParityBit(void)
{
    UART_SFRS->LCTRL.UPARITY = E_UART_PARITY_BIT_ODD;
}

/**
 * @brief Enable UART parity
 */
static __INLINE void f_UART_EnableParity(void)
{
    UART_SFRS->LCTRL.UPAREN = 1;
}

/**
 * @brief Disable UART parity
 */
static __INLINE void f_UART_DisableParity(void)
{
    UART_SFRS->LCTRL.UPAREN = 0;
}

/**
 * @brief Enable UART sticky parity
 */
static __INLINE void f_UART_EnableStickyParity(void)
{
    UART_SFRS->LCTRL.USTICKEN = 1;
}

/**
 * @brief Disable UART sticky parity
 */
static __INLINE void f_UART_DisableStickyParity(void)
{
    UART_SFRS->LCTRL.USTICKEN = 0;
}

/**
 * @brief Enable UART break
 */
static __INLINE void f_UART_EnableBreak(void)
{
    UART_SFRS->LCTRL.UBREAKEN = 1;
}

/**
 * @brief Disable UART break
 */
static __INLINE void f_UART_DisableBreak(void)
{
    UART_SFRS->LCTRL.UBREAKEN = 0;
}

/**
 * @brief Enable UART loop back
 */
static __INLINE void f_UART_EnableLoopBack(void)
{
    UART_SFRS->LCTRL.ULOOPEN = 1;
}

/**
 * @brief Disable UART loop back
 */
static __INLINE void f_UART_DisableLoopBack(void)
{
    UART_SFRS->LCTRL.ULOOPEN = 0;
}

/**
 * @brief Set UART interrupt delay in cycles
 */
static __INLINE void f_UART_SetInterruptDelay(uint8_t delay)
{
}

#endif /* __UART_SFRS_H__ */
