/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main_led.c
 */


#include "adc_device.h"
#include "clock_device.h"
#include "flash_device.h"
#include "gpio_device.h"
#include "irq_device.h"
#include "led_device.h"
#include "pmu_device.h"
#include "systick_device.h"
#include "timer_device.h"
#include "wdt_device.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)

#define TIMER_MS                30000                  /* 1 mS */

/************** Local Variables ***********************************************/
static GPIO_Device_t *gpiod = NULL;
static LED_Device_t *led = NULL;
static SysTick_Device_t *systick = NULL;
static Timer_Device_t *timer0 = NULL;

static uint32_t key_pressed = 0;

/************** Global Variables **********************************************/
/************** Functions *****************************************************/
static void timer0_func(Timer_Device_t *timer)
{
    /* Turn off LED when timer0 expires */
    LED_devOff(led, 0);
}

static void gpio_pin_func(void)
{
    /* Determine if key is pressed */
    if (GPIO_devRead(gpiod, 2) == 0)
        key_pressed = 1;
    else
        key_pressed = 0;
}

/************** MAIN ****************************************************/
void main_led (void)
{
    /* Initialize system clock */
    Clock_devCrystalClockInit();

    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, NULL);

    /* Initialize Timer0 */
    timer0 = Timer_devRequest("TIMER0");
    if(!timer0)
        return;
    /* Register timer0 interrupt */
    Timer_devRegisterIRQ(timer0, (void *)&timer0_func);
    /* Load 1000ms to timer0 */
    Timer_devSetMs(timer0, TIMER_MS * 1000);
    /* Enable timer0 interrupt */
    Timer_devEnableIRQ(timer0);
    /* Start timer0 */
    Timer_devStart(timer0);

    /* Get GPIOD */
    gpiod = GPIO_devRequest("GPIOD");
    if(!gpiod)
        return;
    /* Initialize GPIOD.0 as interrupt input */
    GPIO_devInit(gpiod, 2, GPIO_MODE_INTERRUPT | GPIO_MODE_INPUT |GPIO_MODE_PULLUP);
    /* Register pin toggling interrupt */
    GPIO_devRegisterIRQ(gpiod, (void *)&gpio_pin_func);

    /* LED driver */
    led = LED_devRequest("LED");
    if(!led)
        return;
    /* Set LED light level to 80% */
    LED_devSetLightLevel(led, 0, 80);
    /* Turn on LED */
    LED_devOn(led, 0);
    
    while (1) {
        if (key_pressed == 1) {
            if (GPIO_devRead(gpiod, 2) == 0) {
                /* Turn on LED */
                LED_devOn(led, 0);
                /* Load 500ms to timer0 */
                Timer_devSetMs(timer0, TIMER_MS * 500);
            }
        }
    } /* while (1) */
} /* main() */

/***************************** End of File ************************************/