/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main_pm.c
 */

#include "adc_device.h"
#include "clock_device.h"
#include "flash_device.h"
#include "gpio_device.h"
#include "irq_device.h"
#include "led_device.h"
#include "pmu_device.h"
#include "systick_device.h"
#include "timer_device.h"
#include "wdt_device.h"
#include "pir_device.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)

#define TIMER_MS                30000                  /* 1 mS */

#define ADC_REF_H_DFLT          0x0F
#define ADC_REF_L_DFLT          0x00
#define ADC_REF_G_DFLT          0x0F

/************** Local Variables ***********************************************/
static LED_Device_t *led = NULL;
static SysTick_Device_t *systick = NULL;
static Timer_Device_t *timer1 = NULL;
static GPIO_Device_t *gpiod;
static GPIO_Device_t *gpioe;
static PIR_Device_t *pir;

static uint32_t sleep_allowed_timer = 0;

/************** Global Variables **********************************************/
/************** Functions *****************************************************/
void pir_wakeup_func(PIR_Device_t *pir)
{
}

static void timer1_func(Timer_Device_t *timer)
{
    static uint8_t count;
    
    count++;
    Timer_devSetMs(timer, TIMER_MS * 1000);
    if (count > 10) {
        sleep_allowed_timer = 1;
        count = 0;
    }
}

static void enter_sleep_mode(void)
{
    LED_devOff(led, 0);
    Clock_devRC10KClockInit();
    Clock_devDisableCrystalClock();
    sleep_allowed_timer = 0;
    PIR_devEnableWakeup(pir, (void *)&pir_wakeup_func);
    PMU_devSleepLowPwr(PMU_WAKEUP_TIME_12000);
}

static void exit_sleep_mode(void)
{
    PIR_devDisableWakeup(pir);
    LED_devOn(led, 0);
    Clock_devCrystalClockInit();
    Timer_devSetMs(timer1, TIMER_MS * 1000);
}

/************** MAIN ****************************************************/
void main_pm (void)
{
    uint8_t sleep_allowed_gpio = 0;
        
    /* Initialize system clock */
    Clock_devCrystalClockInit();

    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, NULL);

    timer1 = Timer_devRequest("TIMER1");
    if(!timer1)
        return;

    Timer_devRegisterIRQ(timer1, (void *)&timer1_func);
    Timer_devSetMs(timer1, TIMER_MS * 1000);
    Timer_devEnableIRQ(timer1);
    Timer_devStart(timer1);

    /* Get GPIOE */
    gpioe = GPIO_devRequest("GPIOE");
    if(!gpioe)
        return;
    GPIO_devInit(gpioe, 6, GPIO_MODE_INPUT |GPIO_MODE_PULLUP);

        /* Get GPIOD */
    gpiod = GPIO_devRequest("GPIOD");
    if(!gpiod)
        return;
    /* Initialize GPIOD.7 as interrupt input for PIR */
    GPIO_devInit(gpiod, 7, GPIO_MODE_INPUT);
    GPIO_devInit(gpiod, 3, GPIO_MODE_INPUT);
    GPIO_devInit(gpiod, 4, GPIO_MODE_INPUT);
    GPIO_devInit(gpiod, 5, GPIO_MODE_INPUT);
    GPIO_devInit(gpiod, 6, GPIO_MODE_INPUT);

    /* LED driver */
    led = LED_devRequest("LED");
    if(!led)
        return;
    /* Set LED light level to 80% */
    LED_devSetLightLevel(led, 0, 80);
    /* Turn on LED */
    LED_devOn(led, 0);

    pir = PIR_devRequest();
    if (!pir)
        return;
    PIR_devInit(pir, 24, 3, 5, 0);
    PIR_devDisableWakeup(pir);

    while (1) {
        sleep_allowed_gpio = GPIO_devRead(gpioe, 6);
        if (sleep_allowed_gpio && sleep_allowed_timer) {
            enter_sleep_mode();
            exit_sleep_mode();
        }
   } /* while (1) */
} /* main() */

/***************************** End of File ************************************/