/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main_rtc.c
 */


#include "clock_device.h"
#include "gpio_device.h"
#include "irq_device.h"
#include "led_device.h"
#include "pmu_device.h"
#include "systick_device.h"
#include "rtc_device.h"
#include "timer_device.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)

#define TIMER_MS                30000                  /* 1 mS */

/************** Local Variables ***********************************************/
static GPIO_Device_t *gpiod = NULL;
static LED_Device_t *led = NULL;
static SysTick_Device_t *systick = NULL;
static RTC_Device_t *rtc = NULL;

static uint32_t key_pressed = 0;

/************** Global Variables **********************************************/
/************** Functions *****************************************************/
static void gpio_pin_func(void)
{
    /* Determine if key is pressed */
    if (GPIO_devRead(gpiod, 2) == 0)
        key_pressed = 1;
    else
        key_pressed = 0;
}

static void alarm0_func(void)
{
    /* Turn off LED when alarm0 fires */
    LED_devOn(led, 0);
    SysTick_devDelayMs(systick, 50);
    LED_devOff(led, 0);
}

static void alarm1_func(void)
{
    /* Turn off LED when alarm1 fires */
    LED_devOn(led, 0);
    SysTick_devDelayMs(systick, 50);
    LED_devOff(led, 0);
    SysTick_devDelayMs(systick, 50);
    LED_devOn(led, 0);
    SysTick_devDelayMs(systick, 50);
    LED_devOff(led, 0);
}

/************** MAIN ****************************************************/
void main_rtc (void)
{
    int32_t ret;
    
    /* Initialize system clock */
    Clock_devCrystalClockInit();

    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, NULL);

    /* Get GPIOD */
    gpiod = GPIO_devRequest("GPIOD");
    if(!gpiod)
        return;
    /* Initialize GPIOD.0 as interrupt input */
    GPIO_devInit(gpiod, 2, GPIO_MODE_INTERRUPT | GPIO_MODE_INPUT |GPIO_MODE_PULLUP);
    /* Register pin toggling interrupt */
    GPIO_devRegisterIRQ(gpiod, (void *)&gpio_pin_func);

    /* LED driver */
    led = LED_devRequest("LED");
    if(!led)
        return;
    /* Set LED light level to 80% */
    LED_devSetLightLevel(led, 0, 80);
    /* Turn on LED */
    LED_devOn(led, 0);
    SysTick_devDelayMs(systick, 500);
    LED_devOff(led, 0);

    /* RTC driver */
    rtc = RTC_devRequest();
    if(!rtc)
        return;
    ret = RTC_devInit(rtc);
    if (ret)
        return;

    /* Start a repeated alarm at alarm 0 */
    ret = RTC_devStartAlarm(rtc, RTC_ALARM_0, RTC_ALARM_MODE_REPEAT, 10,
                           (void *)&alarm0_func);
    if (ret)
        return;

    while (1) {
        if (key_pressed == 1) {
            if (GPIO_devRead(gpiod, 2) == 0) {
                /* Start a signle shot alarm after a key press at alarm 1 */
                ret = RTC_devStartAlarm(rtc, RTC_ALARM_1, RTC_ALARM_MODE_SINGLE, 20,
                    (void *)&alarm1_func);

            }
        }
    } /* while (1) */
} /* main() */

/***************************** End of File ************************************/