/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main_rx.c
 * This demo code receives a message from transmitter and print it to UART port.
 */

#include <stdint.h>

#include "clock_device.h"
#include "led_device.h"
#include "pmu_device.h"
#include "systick_device.h"
#include "rx_device.h"
#include "gpio_device.h"
#include "uart_device.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)

#define TIMER_MS                30000                  /* 1 mS */
#define BYTES_TO_RECEIVE        4

/************** Local Variables ***********************************************/
static LED_Device_t *led = NULL;
static SysTick_Device_t *systick = NULL;
static RX_Device_t *rxdev = NULL;
static GPIO_Device_t *gpioe = NULL;

static uint32_t msg_received = 0;

static uint8_t rx_data[RX_BUFF_SIZE];

/************** Global Variables **********************************************/

/************** Local Functions ***********************************************/
static void rx_func(void)
{
    msg_received = 1;
}

/************** Global Functions **********************************************/
void main_rx()
{
    int32_t ret;
    
    /* Initialize system clock */
    Clock_devCrystalClockInit();

    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, NULL);

    /* LED driver */
    led = LED_devRequest("LED");
    if(!led)
        return;
    /* Set LED light level to 80% */
    LED_devSetLightLevel(led, 0, 80);
    /* Turn on LED */
    LED_devOn(led, 0);
    SysTick_devDelayMs(systick, 100);
    LED_devOff(led, 0);

    /* Request RX device */
    rxdev = RX_devRequest();
    if (!rxdev)
        return;
    /* Regiser call back function to handle received message */
    RX_devRegisterCallback(rxdev, (void *)&rx_func);
    /* Init RX device, the default RX device driver Setup is for 433.92MHz reception*/
    RX_devInit(rxdev, BYTES_TO_RECEIVE, DCM_FMT_4BIT);
   
    /* Config GPIO to UART mode */
    GPIO_ConfigUART(E_GPIO_PIN_MODE_ON, E_GPIO_PIN_NOSWAP, E_GPIO_PIN_NOINVERT);

    /* Config GPIO pins used by UART driver */
    gpioe = GPIO_devRequest("GPIOE");
    if (!gpioe)
        return;
    GPIO_devInit(gpioe, 4, GPIO_MODE_INPUT);
    GPIO_devInit(gpioe, 5,
        GPIO_MODE_PULLUP | GPIO_MODE_OUTPUT);
    
    /* Request UART device */
    UART_Device_t *uart = UART_devRequest("UART");
    UART_Dev_LineInitData_t uart_config = {
        .DataSize = 8,
        .StopBits = 1,
        .ParityBit = UART_DEV_PARITY_BIT_ODD,
        .ParityEn = 0,
        .LoopBackEn = 0,
        .BreakEn = 0,
        .StickyParityEn = 0,
    };
    /* Initialize UART device */
    ret = UART_devInit(uart, UART_DEV_BAUDRATE_9600, &uart_config);
    if (ret)
        return;

    while(1)
    {
        if (msg_received) {
            /* The message is saved at end of buffer, so the last byte recived
             is rx_data[RX_BUFF_SIZE - 1] */
            RX_devGetReceivedData(rxdev, (uint8_t *)&rx_data, RX_BUFF_SIZE);

            if (UART_devCheckError(uart))
                UART_devResetDevice(uart);
                /* Send data through UART device */
                UART_devSendByte(uart, (uint8_t *)&rx_data, RX_BUFF_SIZE);

            msg_received = 0;
            LED_devOn(led, 0);
            SysTick_devDelayMs(systick, 100);
            LED_devOff(led, 0);
        }
    } 
}
