/**
 * @copyright 2015 Indie Semiconductor.
 *
 * This file is proprietary to Indie Semiconductor.
 * All rights reserved. Reproduction or distribution, in whole
 * or in part, is forbidden except by express written permission
 * of Indie Semiconductor.
 *
 * @file main_tx.c
 * This demo code sends a message to TX driver upon a key press.
 *
 */

#include <stdint.h>

#include "clock_device.h"
#include "led_device.h"
#include "gpio_device.h"
#include "pmu_device.h"
#include "systick_device.h"
#include "tx_device.h"

/************** Definitions ***************************************************/
#define SYSTICK_MS              (10)                            /* 10ms */
#define SYSTICK_INTERVAL        (E_HWCFG_SYSTEM_CLOCK/SYSTICK_MS)

#define TIMER_MS                30000                  /* 1 mS */

/************** Local Variables ***********************************************/
static GPIO_Device_t *gpiod = NULL;
static LED_Device_t *led = NULL;
static SysTick_Device_t *systick = NULL;

static uint32_t key_pressed = 0;
static uint8_t tx_data[] = {
    0x01,
    0x23,
    0x45,
    0x67,
};

/************** Local Functions ***********************************************/
static void gpio_pin_func(void)
{
    /* Determine if key is pressed */
    if (GPIO_devRead(gpiod, 2) == 0)
        key_pressed = 1;
    else
        key_pressed = 0;
}

/************** Global Functions **********************************************/
void main_tx()
{ 
    /* Initialize system clock */
    Clock_devCrystalClockInit();
    
    /* Initialize SysTick driver */
    systick = SysTick_devRequest();
    if(!systick)
        return;
    SysTick_devInit(systick, SYSTICK_MODE_ENABLE_INTERRUPT, SYSTICK_INTERVAL, NULL);

    /* Get GPIOD */
    gpiod = GPIO_devRequest("GPIOD");
    if(!gpiod)
        return;
    /* Initialize GPIOD.0 as interrupt input */
    GPIO_devInit(gpiod, 2, GPIO_MODE_INTERRUPT | GPIO_MODE_INPUT |GPIO_MODE_PULLUP);
    /* Register pin toggling interrupt */
    GPIO_devRegisterIRQ(gpiod, (void *)&gpio_pin_func);

    /* LED driver */
    led = LED_devRequest("LED");
    if(!led)
        return;
    /* Set LED light level to 80% */
    LED_devSetLightLevel(led, 0, 80);
    /* Turn on LED */
    LED_devOn(led, 0);

    TX_Device_t *tx = TX_devRequest();
    if (!tx)
        return;

    /* Set TX configuration data */
    TX_Config_t tx_cfg = {
        .format = DCM_FMT_3BIT,
    };
    
    /* Init TX driver */
    TX_devInit(tx, &tx_cfg);
    /* Send a message */
    TX_devSendData(tx, (uint8_t *)&tx_data, sizeof(tx_data));
    LED_devOff(led, 0);               

    while (1) {
        if (key_pressed == 1) {
            if (GPIO_devRead(gpiod, 2) == 0) {
                /* Turn on LED */
                LED_devOn(led, 0);
                TX_devSendData(tx, (uint8_t *)&tx_data, sizeof(tx_data));
                while (tx->IsBusy());
                LED_devOff(led, 0);
                SysTick_devDelayMs(systick, 500);
            }
        }
    } /* while (1) */
}


